#ifndef _PTHREADS_FUNC_H_
#define _PTHREADS_FUNC_H_

/*
   *
   * pthreads_func.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <pthread.h>
#include "pthreads_defs.h"


#ifdef _BCOUNT_H_
extern void* refresh_now_window(void* unused);
#endif


/* Threads veriables */
#ifdef _BCOUNT_H_
pthread_t thread_refresh_stats_window_id;
#endif
pthread_t thread_ife_stats_id, thread_ife_speed_id;
pthread_mutex_t mutex_id;

int pthreads_counter = 0; /* threads amount */

/* threads flags */
unsigned short int pthreads_status = FALSE;
unsigned short int shutdown_thread_refresh_stats_window = FALSE;
unsigned short int shutdown_thread_ife_stats = FALSE;
unsigned short int shutdown_thread_ife_speed = FALSE;



/* Fire up all threads */
int pthreads_starting_up(void)
{
	if (pthreads_status == FALSE);
	    {
		/* Change shutdown_thread_* flags to FALSE */
#ifdef _BCOUNT_H_
		shutdown_thread_refresh_stats_window = FALSE;
#endif /* _BCOUNT_H_ */
		shutdown_thread_ife_stats = FALSE;
		shutdown_thread_ife_speed = FALSE;
		
		/* Initializing default `mutex_id' mutex */		
		if (pthread_mutex_init (&mutex_id, NULL) == -1)
		    {
			fprintf (stderr, "Could't create default mutex!\n");

			return(EXIT_FALSE);
		    }
		
		/* Initializing thread `thread_ife_stats_id' */
		if (pthread_create (&thread_ife_stats_id, NULL, 
				    &interface_stats_read, NULL) == -1)
		    {
			fprintf (stderr, "Could't create STATS thread!\n");

			return(EXIT_FALSE);
		    }
		else		
		    pthreads_counter = pthreads_counter + 1;
		
		/* Initializing thread `thread_ife_speed_id' */
		if (pthread_create (&thread_ife_speed_id, NULL, 
				    &interface_speed_measure, NULL) == -1)
		    {
			fprintf (stderr, "Could't create SPEED thread!\n");

			return(EXIT_FALSE);
		    }
		else		
		    pthreads_counter = pthreads_counter + 1;

#ifdef _BCOUNT_H_
		/* Initializing thread `thread_refresh_stats_window_id' */
		if (pthread_create (&thread_refresh_stats_window_id, NULL, 
				    &refresh_now_window, NULL) == -1)
		    {
			fprintf (stderr, "Could't create REFRESH thread!\n");

			return(EXIT_FALSE);
		    }
		else		
		    pthreads_counter = pthreads_counter + 1;

#endif /* _BCOUNT_H_ */

		pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
		
		/* Set pthreads_status flag to TRUE. 
		    All threads is alive :-) */
		pthreads_status = TRUE;

	    }		

	return(EXIT_SUCCESS);
}


int pthreads_shutting_down(unsigned int which_thread)
{
	if (pthreads_status == TRUE);
	    {
		/* Syncing threads. Waiting when threads is stop */
		if (which_thread == STATS_THREAD 
		    || which_thread == ALL_THREADS)
		    {
			if (shutdown_thread_ife_stats != TRUE)
			    {
				shutdown_thread_ife_stats = TRUE;
				pthread_join (thread_ife_stats_id, NULL);
			    }

			if (which_thread == STATS_THREAD)
			    return(EXIT_SUCCESS);
		    }

		if (which_thread == SPEED_THREAD 
		    || which_thread == ALL_THREADS)
		    {
			if (shutdown_thread_ife_speed != TRUE)
			    {
				shutdown_thread_ife_speed = TRUE;
				pthread_join (thread_ife_speed_id, NULL);
			    }

			if (which_thread == SPEED_THREAD)
			    return(EXIT_SUCCESS);
		    }
#ifdef _BCOUNT_H_
		if (which_thread == REFRESH_THREAD
		    || which_thread == ALL_THREADS)
		    {
			if (shutdown_thread_refresh_stats_window != TRUE)
			    {
				shutdown_thread_refresh_stats_window = TRUE;
				pthread_join (thread_refresh_stats_window_id, NULL);
			    }

			if (which_thread == REFRESH_THREAD)
			    return(EXIT_SUCCESS);
		    }
#endif /* _BCOUNT_H_ */

		/* Kill mutex */
		pthread_mutex_destroy (&mutex_id);
		
		/* Set pthreads_status flag to FALSE. 
		    All threads is killed */
		pthreads_status = FALSE;
	    }

	return(EXIT_SUCCESS);
}


/* NIN */

#endif /* _PTHREADS_FUNC_H_ */


