#ifndef _SMTP_CLIENT_H_
#define _SMTP_CLIENT_H_

/*
   *
   * smtp_client.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include "terminal_beep.h"



/* Standard port for MTA */
#define SMTP_PORT	25


void write_to_smtp_debug_file_error(void);
#ifdef _BCOUNT_H_
void smtp_not_found(void);
void send_mail_failed(void);
void send_mail_ok(void);
void send_mail_queued(void);
void send_mail_clear_status(void);
#endif
int smtp_client_init(void);
int connect_to_smtp_server(char *which_data);
int read_answer_status_code(void);
void send_quit_to_smtp_server(void);
void send_rset_to_smtp_server(void);
int send_email_to_smtp_server (const char *from, const char *to, 
				const char *subject, char *which_data);


#ifdef _BCOUNT_H_
/* Status window definition */
WINDOW *mail_status_window;
#endif

FILE *smtp_fp;
char default_smtp_debug_file[100] = "/tmp/smtp_session_debug.log";
char *smtp_debug_file = default_smtp_debug_file;

char smtp_server_name[100];
char *remote_smtp_server = smtp_server_name;

char transmit_buffer[5150], received_buffer[5150];
char get_mail_digit[25], get_mail_address[45];
char smtp_server_answer_code[10];

int received_flag = FALSE, transmitted_flag = FALSE;
int no_more_mail_flag = TRUE, smtp_init_flag = FALSE;
int return_queued_flag = FALSE, smtp_session_debug_flag = FALSE;
int smtp_session_debug_file_opened_flag = FALSE;

/* E-mail counters */
unsigned int email_sent_ok = 0;
unsigned int email_send_error = 0;

/* Socket description */
int smtp_socket = -1;

struct sockaddr_in address;
ssize_t bytes_read;


void write_to_smtp_debug_file_error(void)
{
	syslog (LOG_ERR, "-ERR: Write error to %s", smtp_debug_file);
	syslog (LOG_ERR, "      SMTP session debug file failed!");

	closelog();
}


#ifdef _BCOUNT_H_
void smtp_not_found(void)
{
	wattron (mail_status_window, COLOR_PAIR(4));
	wattron (mail_status_window, A_BLINK);
	mvwaddstr (mail_status_window, 0, 1, "       ");
	mvwaddstr (mail_status_window, 0, 1, "No SMTP");
	wrefresh (mail_status_window);
}


void send_mail_failed(void)
{
	wattron (mail_status_window, COLOR_PAIR(4));
	wattron (mail_status_window, A_BLINK);
	mvwaddstr (mail_status_window, 0, 1, "       ");
	mvwaddstr (mail_status_window, 0, 1, "Fail:-(");
	wrefresh (mail_status_window);
	syslog (LOG_ERR, "-ERR: Sorry Mail to <%s> was not send :-(", 
		get_mail_address);
}


void send_mail_ok(void)
{
    	wattron (mail_status_window, COLOR_PAIR(2));
	wattroff (mail_status_window, A_BLINK);
	mvwaddstr (mail_status_window, 0, 1, "       ");
	mvwaddstr (mail_status_window, 0, 1, "Sent:-)");
	wrefresh (mail_status_window);
	syslog (LOG_ERR, "+OK: Mail to <%s> sent :-)", get_mail_address);
}


void send_mail_queued(void)
{
    	wattron (mail_status_window, COLOR_PAIR(2));
	wattroff (mail_status_window, A_BLINK);
	mvwaddstr (mail_status_window, 0, 1, "       ");
	mvwaddstr (mail_status_window, 0, 1, "Sent-Q!");
	wrefresh (mail_status_window);
	syslog (LOG_ERR, "+OK: Mail to <%s> in SMTP server queue!", 
		get_mail_address);
}


void send_mail_clear_status(void)
{
    	wattron (mail_status_window, COLOR_PAIR(2));
	wattroff (mail_status_window, A_BLINK);
	mvwaddstr (mail_status_window, 0, 1, "       ");
	wrefresh (mail_status_window);
}
#endif


int smtp_client_init(void)
{
	struct servent *smtp_servent;
	struct hostent *smtp_server;

	/* ! This is for bug report ! */
	strcpy (running_function, "smtp_client_init()");
 
	bzero ((char *) &address, sizeof(address));
	address.sin_family = AF_INET;
 
	/* Open log file */
	openlog (exec_name, LOG_PID, LOG_DAEMON);

#ifdef _BCOUNT_H_
	printf ("+ Searching for SMTP Server...\n");
#endif
	smtp_server = gethostbyname (remote_smtp_server);
	if (smtp_server == NULL)
	    {
		syslog (LOG_ERR, "-ERR: %s: smtp_client_init() -> "
			"gethostbyname()", strerror(h_errno));
#ifdef _BCOUNT_H_
		printf ("    -> Finding host: %s "_STRING_RED_"\n", 
			remote_smtp_server, STRING_FAIL);
		printf ("       Please try other SMTP server otherwise you can't"
			" sending any e-mail\n\n");
		
		press_any_key();
#endif
		smtp_init_flag = FALSE;
		no_more_mail_flag = TRUE;
		closelog();

		return(EXIT_FALSE); 
	    }
	else if (smtp_server != NULL)
	    {	
		bcopy (smtp_server->h_addr, 
			(char *) &address.sin_addr, smtp_server->h_length);
#ifdef _BCOUNT_H_
		printf ("    -> Finding host: %s "_STRING_WHITE_"\n", 
			remote_smtp_server, STRING_OK);
#endif
    	    }
       
	smtp_servent = getservbyport (htons(SMTP_PORT), "tcp");
	if (smtp_servent == NULL)
	    {
		syslog (LOG_ERR, "-ERR: %s: smtp_client_init() -> "
			"getservbyport()", strerror(h_errno));
#ifdef _BCOUNT_H_
		printf ("    -> Opening port on host: %d "_STRING_RED_"\n"
			, SMTP_PORT, STRING_FAIL);
		printf ("       Please try other SMTP server othrwise you can't"
			" sending any e-mail\n\n");
		
		press_any_key();
#endif
		smtp_init_flag = FALSE;
		no_more_mail_flag = TRUE;
		closelog();

		return(EXIT_FALSE); 
	    
	    }
	else if (smtp_servent != NULL)
		address.sin_port = smtp_servent->s_port;


	/* Testing connection with SMTP server on dedicated port */
	smtp_socket = inet_socket_stream("smtp_client_init()");
	if (smtp_socket < 0)
	    {
#ifdef _BCOUNT_H_
		press_any_key();
#endif
		smtp_init_flag = FALSE;
		no_more_mail_flag = TRUE;
		closelog();

		return(EXIT_FALSE);
	    }

	if (connect (smtp_socket, (struct sockaddr *) &address, 
			sizeof(address)) < 0)
	    {
#ifdef _BCOUNT_H_
		printf ("    -> Connection on port: %d "_STRING_RED_"\n", 
			SMTP_PORT, STRING_FAIL);

		if (errno == ECONNREFUSED)
		    printf ("       Connection refused or server not "
			    "running on this port\n");

		if (errno == ECONNRESET)
		    printf ("       Connection reset. Connection reset "
			    "through other proces\n");

		if (errno == ECONNABORTED)
		    printf ("       Connection aborted. I don't "
			    "known why!?\n");

		printf ("       Please try other SMTP server otherwise you can't"
			" sending any e-mail\n\n");
		
		press_any_key();
#endif

		smtp_init_flag = FALSE;
		no_more_mail_flag = TRUE;
		closelog();
		
		close(smtp_socket);
		
		return(EXIT_FALSE);
	    }
	else
	    {
#ifdef _BCOUNT_H_
		printf ("    -> Connection on port: %d "_STRING_WHITE_"\n", 
			SMTP_PORT, STRING_OK);
#endif
		/* Say goodbye to SMTP server */
		snprintf (transmit_buffer, sizeof(transmit_buffer), "RSET"CRLF);
		write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

		snprintf (transmit_buffer, sizeof(transmit_buffer), "QUIT"CRLF);
		write (smtp_socket, transmit_buffer, strlen(transmit_buffer));
	    }

	/* We made connection with SMTP server */
	smtp_init_flag = TRUE;

	closelog();
	
	/* Close socket */
	close(smtp_socket);

#ifdef _BCOUNT_H_
	/* Sleeping 3sec. */
	sleep(3);
#endif
	return(EXIT_SUCCESS); 
}


int connect_to_smtp_server(char *which_data)
{
	char mail_from[50];

	/* ! This is for bug report ! */
	strcpy (running_function, "connect_to_smtp_server()");
 
	if (smtp_init_flag == FALSE)
	    {
#ifdef _BCOUNT_H_
		if (curses_flag == TRUE)
		    smtp_not_found();
#endif
		return(EXIT_FALSE);
	    }

	/* Open the log file */
	openlog (exec_name, LOG_PID, LOG_DAEMON);

	smtp_socket = socket (PF_INET, SOCK_STREAM, 0);
	if (smtp_socket == -1)
	    syslog (LOG_ERR, "-ERR: %s: connect_to_smtp_server() -> socket()",
		    strerror(errno));

	if (connect (smtp_socket, (struct sockaddr *) &address, 
			sizeof(address)) < 0)
	    {
		syslog (LOG_ERR, "-ERR: %s: connect_to_smtp_server() -> "
			"connect()", strerror(h_errno));
		if (errno == ECONNREFUSED)
		    syslog (LOG_ERR, "-ERR: Connection refused or server not "
			    "running on this port");

		if (errno == ECONNRESET)
		    syslog (LOG_ERR, "-ERR: Connection reset. Connection reset "
			    "through other proces");

		if (errno == ECONNABORTED)
		    syslog (LOG_ERR, "-ERR: Connection aborted. I don't "
			    "known why!?");

#ifdef _BCOUNT_H_
		if (curses_flag == TRUE)
		    send_mail_failed();
#endif
		no_more_mail_flag = TRUE;
		closelog();
		
		close(smtp_socket);
		
		return(EXIT_FALSE);
	    }

	/* If we want debug session with SMTP Server 
	    then open the debug file */
	if (smtp_session_debug_flag == TRUE)
	    {
		if ((smtp_fp = fopen (smtp_debug_file, "a")) == NULL)
		    write_to_smtp_debug_file_error();
		
		smtp_session_debug_file_opened_flag = TRUE;
		
		fprintf (smtp_fp, "\n\n#\n## %s %s Session with %s SMTP "
			"Server\n#\n", get_localdate(), get_localtime(), 
			remote_smtp_server);

		fflush(smtp_fp);
	    }

	if (smtp_session_debug_flag == TRUE)
	    {
		/* Connection with SMTP server */
		snprintf (transmit_buffer, sizeof(transmit_buffer), 
			"(connection)"CRLF);
	    }
	    
	/* Read 220 welcome answers from server. Welcome SMTP server message */
	if (read_answer_status_code() == FALSE)
	    {
		syslog (LOG_ERR, "-ERR: SMTP Server don't send 220 "
			"answer code. First message from SMTP Server");
		no_more_mail_flag = TRUE;
		closelog();
		
		close(smtp_socket);

#ifdef _BCOUNT_H_
		if (curses_flag == TRUE)
		    send_mail_failed();
#endif
		return(EXIT_FALSE);
	    }

	/* We doing email sender address */
	strcpy (mail_from, exec_name);
	strcat (mail_from, "@"); /* At */
	strcat (mail_from, local_host_name);

	/* Send e-mail to SMTP server */
    	check_exit = (send_email_to_smtp_server (mail_from, get_mail_address, 
				    "From Bytes Counter", which_data));

	/* This is importent if we can't get more mail :-) */
	no_more_mail_flag = TRUE;

	closelog();

	/* Close socket */
	close(smtp_socket);

	/* SMTP server sent e-mail successful */
	if (check_exit == EXIT_SUCCESS)
	    {
#ifdef _BCOUNT_H_
		if (curses_flag == TRUE)
		    send_mail_ok();

		email_sent_ok++;
#endif
		terminal_beep();
	    }
	/* SMTP server was not send e-mail */
	else if (check_exit == EXIT_FALSE)
	    {
#ifdef _BCOUNT_H_
		if (curses_flag == TRUE)
		    send_mail_failed();

		email_send_error++;
#endif		
		return(EXIT_FALSE);
	    }
	/* SMTP server can't send e-mail in this moment, but keep it in queue */
	else if (check_exit == EXIT_QUEUED)
	    {
#ifdef _BCOUNT_H_
		if (curses_flag == TRUE)
		    send_mail_queued();

		email_sent_ok++;
#endif
		terminal_beep();
	    
		/* Reset the return_queued_flag */
		return_queued_flag = FALSE;
		return(EXIT_QUEUED);
	    }
	    

	return(EXIT_SUCCESS);
}


int read_answer_status_code(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "read_answer_status_code()");
 
	bytes_read = read (smtp_socket, received_buffer, 
				    sizeof(received_buffer));
	if (bytes_read > 0)
	    {
		/* add NULL byte */
		received_buffer[bytes_read] = '\0';
    
		sscanf (received_buffer, "%s", smtp_server_answer_code);

		/* Debug session if we need this */
		if (smtp_session_debug_flag == TRUE)
		    {
			fprintf (smtp_fp, "<C>: %s", transmit_buffer);
			fprintf (smtp_fp, "<S>: %s", received_buffer);
			
			if (smtp_session_debug_file_opened_flag == TRUE)
			    fflush(smtp_fp);

		    }	    

		/* *** Checking good answers from server *** */

		/* 220 SMTP Answer Code. First message from SMTP Server */
		if (!strcmp (smtp_server_answer_code, "220"))
		    {
			syslog (LOG_ERR, "+OK [%s answer code]: Opened "
				"connection with SMTP Server: %s", 
				smtp_server_answer_code, remote_smtp_server);

			return(EXIT_SUCCESS);
		    }
		/* 221 SMTP Answer Code. Close connection */
		else if (!strcmp (smtp_server_answer_code, "221"))
		    {
			syslog (LOG_ERR, "+OK [%s answer code]: Server %s "
				"closed connection", smtp_server_answer_code, 
				remote_smtp_server);

			return(EXIT_SUCCESS);
		    }
		/* 250 SMTP Answer Code. Requested mail action ok, completed */
		else if (!strcmp (smtp_server_answer_code, "250"))
		    {
			return(EXIT_SUCCESS);
		    }
		/* 251 SMTP Answer Code. User not local. Automatic forward */
		else if (!strcmp (smtp_server_answer_code, "251"))
		    {
			syslog (LOG_ERR, "+OK [%s answer code]: User not local. "
				"Automatic forward", smtp_server_answer_code);

			return(EXIT_SUCCESS);
		    }
		/* 354 SMTP Answer Code. Info from server about email send */
		else if (!strcmp (smtp_server_answer_code, "354"))
		    {
			return(EXIT_SUCCESS);
		    }

		/* *** Checking error answers from server *** */

		/* 421 SMTP Answer Code. Service not available */
		if (!strcmp (smtp_server_answer_code, "421"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Service not "
				"available, closing transmission channel", 
				smtp_server_answer_code);
    			syslog (LOG_ERR, "     Please try later");
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 450 SMTP Answer Code. Mailbox anavailable */
		else if (!strcmp (smtp_server_answer_code, "450"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Requested mail "
				"action not taken. Mailbox anavailable", 
				smtp_server_answer_code);
    			syslog (LOG_ERR, "     Mabey mailbox busy?!");
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 500 SMTP Answer Code. Syntax error */
		else if (!strcmp (smtp_server_answer_code, "500"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Syntax error, "
				"command unrecognized", smtp_server_answer_code);
    			syslog (LOG_ERR, "     Why we send unknown command "
				"to SMTP server?!");
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 501 SMTP Answer Code. Syntax error in parameters */
		else if (!strcmp (smtp_server_answer_code, "501"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Syntax error "
				"in parameters or arguments", 
				smtp_server_answer_code);
    			syslog (LOG_ERR, "     Why we send unknown parameters "
				"to server?!");
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 502 SMTP Answer Code. Command not implemented */
		else if (!strcmp (smtp_server_answer_code, "502"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Command "
				"not implemented", smtp_server_answer_code);
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 503 SMTP Answer Code. Bad sequence of commands */
		else if (!strcmp (smtp_server_answer_code, "503"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Bad sequence "
				"of commands", smtp_server_answer_code);
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 504 SMTP Answer Code. Command parameter not implemented */
		else if (!strcmp (smtp_server_answer_code, "504"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Command "
				"parameter not implemented", 
				smtp_server_answer_code);
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 550 SMTP Answer Code. Maibox not found */
		else if (!strcmp (smtp_server_answer_code, "550"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Requested "
			    "action not taken, mailbox anavailable.", 
			    smtp_server_answer_code);
    			syslog (LOG_ERR, "     Mailbox not found or no access");
    			syslog (LOG_ERR, "     Please check if you enter "
				"properly e-mail address");
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 551 SMTP Answer Code. User not local. Non automatic forward */
		else if (!strcmp (smtp_server_answer_code, "551"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: User not local. "
				"Non automatic forward", smtp_server_answer_code);
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* 553 SMTP Answer Code. Maibox name not allowed */
		else if (!strcmp (smtp_server_answer_code, "553"))
		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: Requested "
			    "action not taken, maibox name not allowed. ", 
			    smtp_server_answer_code);
    			syslog (LOG_ERR, "     Mailbox syntax incorrect");
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		/* Others codes */
		else
    		    {
			syslog (LOG_ERR, "-ERR [%s answer code]: ?",
				smtp_server_answer_code);
			syslog (LOG_ERR, "     This is received answer from server:");
			syslog (LOG_ERR, "     %s", received_buffer);

	    		send_rset_to_smtp_server();
	    		send_quit_to_smtp_server();

			return(EXIT_FALSE);
		    }
		    
	    }  
	else if (bytes_read < 0)
	    {
		syslog (LOG_ERR, "-ERR: %s: read_answer_status_code() -> "
			"read()", strerror(h_errno));
	    	syslog (LOG_ERR, "      I can't get any answer from SMTP "
			"Server!");

		no_more_mail_flag = TRUE;
		closelog();
		close(smtp_socket);

		return(EXIT_FALSE);
	    }

	return(EXIT_SUCCESS);
}


void send_quit_to_smtp_server(void)
{
	snprintf (transmit_buffer, sizeof(transmit_buffer), "QUIT"CRLF);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	bytes_read = read (smtp_socket, received_buffer, 
				    sizeof(received_buffer));
	if (bytes_read > 0)
	    {
		/* add NULL byte */
		received_buffer[bytes_read] = '\0';
    
		/* Debug session if we need this */
		if (smtp_session_debug_flag == TRUE)
		    {
			fprintf (smtp_fp, "<C>: %s", transmit_buffer);
			fprintf (smtp_fp, "<S>: %s", received_buffer);

			if (smtp_session_debug_file_opened_flag == TRUE)
			    fflush(smtp_fp);

		    }	    
	    }

}


void send_rset_to_smtp_server(void)
{
	snprintf (transmit_buffer, sizeof(transmit_buffer), "RSET"CRLF);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	bytes_read = read (smtp_socket, received_buffer, 
				    sizeof(received_buffer));
	if (bytes_read > 0)
	    {
		/* add NULL byte */
		received_buffer[bytes_read] = '\0';
    
		/* Debug session if we need this */
		if (smtp_session_debug_flag == TRUE)
		    {
			fprintf (smtp_fp, "<C>: %s", transmit_buffer);
			fprintf (smtp_fp, "<S>: %s", received_buffer);

			if (smtp_session_debug_file_opened_flag == TRUE)
			    fflush(smtp_fp);

		    }	    
	    }

}


int send_email_to_smtp_server (const char *from, const char *to, 
				const char *subject, char *which_data)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "send_email_to_smtp_server()");
 
	/* Send `HELO' with local host name to SMTP Server */
	snprintf (transmit_buffer, sizeof(transmit_buffer), 
		    "HELO %s"CRLF, local_host_name);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	if (read_answer_status_code() == FALSE)
	    {
	    	send_quit_to_smtp_server();

		syslog (LOG_ERR, "     Calling with: "
			"send_email_to_smtp_server() -> HELO");

		no_more_mail_flag = TRUE;

		return(EXIT_FALSE);
	    }

	/* Send `Reverse-Path' to server */
	snprintf (transmit_buffer, sizeof(transmit_buffer), 
		    "MAIL FROM:<%s>"CRLF, from);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	if (read_answer_status_code() == FALSE)
	    {
	    	send_quit_to_smtp_server();

	    	syslog (LOG_ERR, "     Calling with: "
			"send_email_to_smtp_server() -> MAIL FROM");
	    	syslog (LOG_ERR, "     Mail from: %s? if this is not "
			"correct address, please send info to %s", 
			from, email_address);

		no_more_mail_flag = TRUE;

		return(EXIT_FALSE);
	    }

	/* Send e-mail address */
	snprintf (transmit_buffer, sizeof(transmit_buffer), 
		    "RCPT TO:<%s>"CRLF, to);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	if (read_answer_status_code() == FALSE)
	    {
	    	send_quit_to_smtp_server();

    		syslog (LOG_ERR, "     Calling with: "
			"send_email_to_smtp_server() -> RCPT TO");

		no_more_mail_flag = TRUE;

		return(EXIT_FALSE);
	    }

	/* Check if email is Queued */
	if (strstr (received_buffer, "queue") != NULL)
	    return_queued_flag = TRUE;

	/* Send `DATA' info */
	snprintf (transmit_buffer, sizeof(transmit_buffer), "DATA"CRLF);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	if (read_answer_status_code() == FALSE)
	    {
	    	send_quit_to_smtp_server();
	    	syslog (LOG_ERR, "     Calling with: "
			"send_email_to_smtp_server() -> DATA");

		no_more_mail_flag = TRUE;

		return(EXIT_FALSE);
	    }

	/* Sending all content (header + body) */
#ifdef _BCOUNT_H_
	if (!strcmp (which_data, "sent"))
	    snprintf(transmit_buffer, sizeof(transmit_buffer), 
		    "From: %s"LF
		    "To: %s"LF
		    "Subject: %s"LF
		    "Content-Type: text/plain"LF
		    "X-Priority: 3 (Normal)"LF
		    LF
		    "Hi ;-)"LF
		    "Your %s interface %s: %.1f MB"LF
		    "Of course at last interface shutdown :-)"LF
		    LF
		    "Regards"LF
		    LF, 
		    from, to, subject, i->name, 
		    which_data, i->stats.tx_mbytes);

	if (!strcmp (which_data, "get"))
		snprintf(transmit_buffer, sizeof(transmit_buffer), 
		    "From: %s"LF
		    "To: %s"LF
		    "Subject: %s"LF
		    "Content-Type: text/plain"LF
		    "X-Priority: 3 (Normal)"LF
		    LF
		    "Hi ;-)"LF
		    "Your %s interface %s: %.1f MB"LF
		    "Of course at last interface shutdown :-)"LF
		    LF
		    "Regards"LF
		    LF, 
		    from, to, subject, i->name, 
		    which_data, i->stats.rx_mbytes);

	if (!strcmp (which_data, "core"))
	    {
		snprintf(transmit_buffer, sizeof(transmit_buffer), 
		    "From: %s"LF
		    "To: %s"LF
		    "Subject: %s [ AUTO BUG REPORT :-( ]"LF
		    "Content-Type: text/plain"LF
		    "X-Priority: 1"LF
		    LF LF
		    "  Program name: %s"LF
		    "  Program version: %s"LF
		    "  Run with user rights: %s"LF
		    "  Examined Interface: %s"LF
		    "  Host name: %s"LF
		    "  Machine class: %s"LF
		    "  Kernel release: %s"LF
		    "  Called function: %s"LF LF,
		    from, to, subject, exec_name, VERSION, 
		    get_user_name(getuid()), i->name, local_host_name, 
		    get_local_host_info(HOST_ARCHITECTURE),
		    get_local_host_info(HOST_KERNEL_RELEASE), error_function);

		free(u_name);
	    }
#endif

#ifdef _BSERVER_H_
	if (!strcmp (which_data, "core"))
	    {
		snprintf(transmit_buffer, sizeof(transmit_buffer), 
		    "From: %s"LF
		    "To: %s"LF
		    "Subject: %s [ AUTO BUG REPORT :-( ]"LF
		    "Content-Type: text/plain"LF
		    "X-Priority: 1"LF
		    LF LF
		    "  Program name: %s"LF
		    "  Program version: %s"LF
		    "  Run with user rights: %s"LF
		    "  Examined Interface: %s"LF
		    "  Host name: %s"LF
		    "  Machine class: %s"LF
		    "  Kernel release: %s"LF
		    "  Called function: %s"LF LF,
		    from, to, subject, exec_name, SVERSION, 
		    get_user_name(getuid()), i->name, local_host_name, 
		    get_local_host_info(HOST_ARCHITECTURE),
		    get_local_host_info(HOST_KERNEL_RELEASE), error_function);

		free(u_name);
	    }
#endif

	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	if (smtp_session_debug_flag == TRUE)
	    {
		fprintf (smtp_fp, "<C>:\n%s", transmit_buffer);

		if (smtp_session_debug_file_opened_flag == TRUE)
		    fflush(smtp_fp);

	    }	    

	snprintf (transmit_buffer, sizeof(transmit_buffer), CRLF"."CRLF);
	write (smtp_socket, transmit_buffer, strlen(transmit_buffer));

	if (read_answer_status_code() == FALSE)
	    {
	    	send_quit_to_smtp_server();

	    	syslog (LOG_ERR, "     Calling with: "
			"send_email_to_smtp_server() -> . (DOT)");
	    	syslog (LOG_ERR, "     Can't end email with \".\" on a "
			"line by itself");

		no_more_mail_flag = TRUE;

		return(EXIT_FALSE);
	    }

	/* Closing connection with server */
    	send_quit_to_smtp_server();

	/* Close debug file if debug session is opened */
	if (smtp_session_debug_flag == TRUE)
	    {
		if (smtp_session_debug_file_opened_flag == TRUE)
		    {
			fflush(smtp_fp);
			fclose(smtp_fp);
		    }

		smtp_session_debug_file_opened_flag = FALSE;
	    }
	    
	if (return_queued_flag == TRUE)
	    return(EXIT_QUEUED);
	    
	return(EXIT_SUCCESS);
}



/* NIN */

#endif /* _SMTP_CLIENT_H_ */

