#ifndef _SYS_INFO_H_
#define _SYS_INFO_H_

/*
   *
   * sys_info.h - Information about CPU speed & free memory
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <pwd.h>
#include <sys/resource.h>
#include "xstrdup.h"


#define PROC_CPUINFO	"/proc/cpuinfo"
#define PROC_MEMINFO	"/proc/meminfo"
#define PROC_STATM	"/proc/%d/statm"

#define GET_FULL_NAME_OK	0
#define GET_FULL_NAME_FALSE	1



struct passwd *psd;
char sys_buffer[1024];
char* match;	
int max_rss_mem = 0;


float get_cpu_clock_speed(void)
{
	FILE* cpu_info;
	float cpu_clock_speed;	
	size_t bytes_read;
	
	cpu_info = fopen (PROC_CPUINFO, "r");
	bytes_read = fread (sys_buffer, 1, sizeof(sys_buffer), cpu_info);
	fclose(cpu_info);
	if (bytes_read == 0 || bytes_read == sizeof(sys_buffer))
	    exit_prog(EXIT_FALSE);
	
	sys_buffer[bytes_read] = '\0';
	
	match = strstr(sys_buffer, "cpu MHz");
	if (match == NULL)
	    return(EXIT_FALSE);
	sscanf (match, "cpu MHz : %f", &cpu_clock_speed);

	return cpu_clock_speed;
}


int get_free_memory(void)
{
	FILE* mem_info;
	int free_memory;	
	size_t bytes_read;
	
	mem_info = fopen (PROC_MEMINFO, "r");
	bytes_read = fread (sys_buffer, 1, sizeof(sys_buffer), mem_info);
	fclose(mem_info);
	if (bytes_read == 0 || bytes_read == sizeof(sys_buffer))
	    exit_prog(EXIT_FALSE);
	
	sys_buffer[bytes_read] = '\0';
	
	match = strstr(sys_buffer, "MemFree");
	if (match == NULL)
	    return(EXIT_FALSE);
	sscanf (match, "MemFree: %d", &free_memory);

	return free_memory;
}


static int get_rss_memory(pid_t pid)
{
	int mem_fd; 
	char file_name[64];
	char mem_info[128];
	int rss = 0;

	snprintf (file_name, sizeof(file_name), PROC_STATM, (int) pid);
	
	if ((mem_fd = open (file_name, O_RDONLY)) == -1)
	    return(EXIT_FALSE);
	
	if ((read (mem_fd, mem_info, sizeof(mem_info) - 1)) <= 0)
	    return(EXIT_FALSE);

	close(mem_fd);
	
	mem_info[strlen(mem_info)] = '\0';
	
	if ((sscanf (mem_info, "%*d %d", &rss)) != 1)
	    return(EXIT_FALSE);
	
	if ((rss * getpagesize()) / 1024 > max_rss_mem)
	    max_rss_mem = (rss * getpagesize()) / 1024;
	return (rss * getpagesize()) / 1024;
}


#ifdef _BCOUNT_H_
void print_process_stats(void)
{
	struct rusage *ru = (struct rusage *) xmalloc (sizeof(struct rusage));

	getrusage(RUSAGE_SELF, ru);
	printf ("CPU time: "_COLOR_WHITE_"%ld.%06ld"_COLOR_END_" sec. [user] | "
		_COLOR_WHITE_"%ld.%06ld"_COLOR_END_" sec. [sys]\n"
		"MEM used: "_COLOR_WHITE_"%d"_COLOR_END_" KB [max rss]\n"
		"SIGNAL's: "_COLOR_WHITE_"%d"_COLOR_END_" [amount]\n",
		ru->ru_utime.tv_sec, ru->ru_utime.tv_usec, ru->ru_stime.tv_sec, 
		ru->ru_stime.tv_usec, max_rss_mem, 
		quantity_signals - pthreads_counter);
	
	free(ru);
}
#endif


/*
    int resource =
		    RLIMIT_CPU
		    RLIMIT_DATA
		    RLIMIT_FSIZE
		    RLIMIT_LOCKS
		    RLIMIT_MEMLOCK
		    RLIMIT_NOFILE
		    RLIMIT_NPROC
		    RLIMIT_RSS
		    RLIMIT_STACK
*/
void get_rlimits_for_process(int resource)
{
	struct rlimit *rlim = (struct rlimit *) 
			    xmalloc (sizeof(struct rlimit));

	getrlimit(resource, rlim);
	
	fprintf (stdout, "Resource [%d]:\n", resource);
	fprintf (stdout, "rlim_cur = %d\n", (int) rlim->rlim_cur);
	fprintf (stdout, "rlim_max = %d\n", (int) rlim->rlim_max);
	fprintf (stdout, "errno = %d [%s]\n\n", errno, strerror(errno));
	
	free(rlim);
}


char* get_user_name(uid_t uid)
{
	static char username[10];
	
	if ((psd = getpwuid (uid)) == NULL)
	    exit_prog(EXIT_FALSE);
		
	strcpy (username, (xstrdup (psd->pw_name)));
		
	return username;
}


/* NIN */

#endif /* _SYS_INFO_H_ */

