/*
   *
   * bclient.c - Bytes Counter simple Client
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include "bclient.h"
#include "fade_screen.h"
#include "get_request.h"
#include "clear_buffer.h"
#include "rotate_dash.h"
#include "license.h"



int main(int argc, char **argv, char **envp)
{
	int c;
	fd_set readfd;
	struct timeval timeout;

	/* Set default terminal as "linux" -> TERM=linux */
	**envp = set_terminal_type(TERM_LINUX);

	exec_name = strrchr (argv[0], '/');
	if (!exec_name)
	    exec_name = argv[0];
	else
	    ++exec_name;


	while ((c = getopt_long(argc, argv, "s:i:p:d:m:lvVh", 
				longopts, 0)) != EOF)
    	    switch (c)
        	{
		    case 's':
			strcpy (server_name, optarg);
			break;
		    case 'i':
			strcpy (server_interface, optarg);
			break;
		    case 'p':
			server_port = atoi(optarg);
			if (server_port == 0)
			    port_error();
			break;
		    case 'd':
			strcpy(data_type, optarg);
			check_data_type();
			break;
		    case 'm':
			if (strlen(optarg) > 62)
			    message_error();
			strcpy(message, optarg);
			break;
		    case 'v':
		    case 'V':
			print_client_version();
		    case 'l':
			print_license();
			break;
		    case 'h':
			print_client_usage();
			exit(EXIT_SUCCESS);
			break;
		    default:
			print_info();
		}

	/* Print Client version */
	print_welcome();

	/* If we can't put server name with -s option then print error */
	if (!strcmp (server_name, ANULL))
	    server_name_error();
	    
	/* Initialize Client */
	client_init();
		
	/* Initialize Curses */
	init_ncurses_screen();

	/* Connecting to server */
	connect_to_server();

	/* Drawing Windows, etc. */
	draw_ncurses_screen();


	do
	    {
		FD_ZERO (&readfd);
		FD_SET (0, &readfd);
		timeout.tv_sec = 1;
		timeout.tv_usec = 0;

		switch ((select (1, &readfd, NULL, NULL, &timeout)) > 0)
		    {
			case 0:
			    if ((FD_ISSET (0, &readfd)) == 0)
				{
				    move(2, 36);
				    refresh_server_data_window();
				    rotate_dash(server_data_window);
				}
			    break;
			case -1:
			    if (errno != EINTR)
				{
				    fprintf(stderr, _STRING_RED_": "
					    "main() -> select()\n", 
					    strerror(errno));

				    exit(EXIT_FALSE);
				}
			    else
				continue;
			    break;
			case 1:
			case 2:
			    if (FD_ISSET (0, &readfd))
				{
				    switch (c = getch())
					{
					    case 'q':
					    case 'Q':
						break;
					    default:
						break;
					}

				}
			    break;
		    }

	    }
	while (tolower(c) != 'q');

	close_ncurses_screen();
	close(csd);
	printf ("Close conection with server.\n"
		"Exit.\n");

	return(EXIT_SUCCESS);
}



/* Initialize Curses */
void init_ncurses_screen(void)
{
	int y_max, x_max;

	main_window = initscr();
	getmaxyx (stdscr, y_max, x_max);

	server_data_window = derwin (stdscr, y_max - 14, x_max - 45, 2, 22);
	getmaxyx (server_data_window, server_y_max, server_x_max);

	info_window = derwin (stdscr, y_max - 19, x_max - 41, 15, 20);
	message_window = derwin (stdscr, 0, x_max - 10, y_max - 1, 0);
	version_window = derwin (stdscr, 0, x_max - 70, y_max - 1, 70);
	connect_window = derwin (stdscr, 6, 43, 9, 18);

	cbreak();
	noecho();

	/* Initialize Curses Color */
	init_ncurses_screen_color();
}


/* Window border, fill, text, etc. ... */
void draw_ncurses_screen(void)
{
	curs_set(INVISIBLE);
	wattron (version_window, COLOR_PAIR(1));
	fill_panel (version_window);
	mvwaddch (version_window, 0, 0, ACS_VLINE);
	mvwprintw (version_window, 0, 1, " Ver %s", CVERSION);
	wrefresh (version_window);

	refresh();

	wattron (connect_window, COLOR_PAIR(1));
	box (connect_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (connect_window, " Connect to . . . ", TOP);
	fill_window (connect_window);
	mvwaddstr (connect_window, 2, 2, "Server name or IP: alpha.deth.org.pl");
	mvwaddstr (connect_window, 3, 2, "Server port:");
	wrefresh (connect_window);

	getch();
	
	delwin (connect_window);
	wrefresh (stdscr);
	wrefresh (main_window);
	
	refresh();

	wattron (server_data_window, COLOR_PAIR(1));
	box (server_data_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (server_data_window, " Server RX/TX: ", TOP);
	fill_window (server_data_window);
	mvwaddstr (server_data_window, 2, 3, "Transmited Data:");
	mvwaddstr (server_data_window, 3, 11, "Packets:");
	mvwaddstr (server_data_window, 4, 12, "Errors:");
	mvwaddstr (server_data_window, 5, 5, "Received Data:");
	mvwaddstr (server_data_window, 6, 11, "Packets:");
	mvwaddstr (server_data_window, 7, 12, "Errors:");
	mvwaddstr (server_data_window, 8, 4, "Content length:");
	wrefresh (server_data_window);

	wattron (info_window, COLOR_PAIR(1));
	box (info_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (info_window, " Info ", TOP);
	fill_window (info_window);
	mvwprintw (info_window, 1, 2, "Server: %s:%d", 
		    server_name, server_port);
	mvwprintw (info_window, 2, 2, "Server ver.: %s", server_version);
	mvwprintw (info_window, 3, 2, "Interface: %s", server_interface);
	mvwprintw (info_window, 4, 2, "Interface IP: %s", server_interface_ip);
	wrefresh (info_window);

	wattron (message_window, COLOR_PAIR(1));
	fill_panel (message_window);
	mvwprintw (message_window, 0, 0, " Msg: %s", server_message);
	wrefresh (message_window);

	refresh();
}


/* Close initialized Windows */
void close_ncurses_screen(void)
{
	delwin (main_window);
	delwin (server_data_window);
	delwin (info_window);
	delwin (message_window);
	delwin (version_window);
	erase();
	refresh();
	endwin();
}


/* Refresh data received from Server */
void refresh_server_data_window(void)
{
	/* Connect to server and get data */
	connect_to_server();

	wattron (server_data_window, A_UNDERLINE);
	mvwaddstr (server_data_window, 2, 20, "              ");
	mvwaddstr (server_data_window, 3, 20, "              ");
	mvwaddstr (server_data_window, 4, 20, "              ");
	mvwaddstr (server_data_window, 5, 20, "              ");
	mvwaddstr (server_data_window, 6, 20, "              ");
	mvwaddstr (server_data_window, 7, 20, "              ");
	mvwaddstr (server_data_window, 8, 20, "              ");
    	mvwprintw (server_data_window, 2, 20, "%s %s", 
		    server_bytes_tx, data_type);
	mvwprintw (server_data_window, 3, 20, "%s", server_packets_tx);
	mvwprintw (server_data_window, 4, 20, "%s", server_errs_tx);
	mvwaddstr (server_data_window, 5, 20, "              ");
	mvwprintw (server_data_window, 5, 20, "%s %s", 
		    server_bytes_rx, data_type);
	mvwprintw (server_data_window, 6, 20, "%s", server_packets_rx);
	mvwprintw (server_data_window, 7, 20, "%s", server_errs_rx);
	mvwprintw (server_data_window, 8, 20, "%d Bytes", content_length);
	wattron (server_data_window, A_NORMAL);
	wrefresh (server_data_window);

	refresh();
}


int client_init(void)
{
	struct servent *serv;
	struct hostent *remote_server;
 
	bzero ((char *) &address, sizeof(address));
	address.sin_family = AF_INET;
 
	remote_server = gethostbyname (server_name);
	if (remote_server != NULL)
	    bcopy (remote_server->h_addr, 
		    (char *) &address.sin_addr, remote_server->h_length);
	else
	    {
		fprintf(stderr, _STRING_RED_": client_init() -> "
			"gethostbyname()\n", strerror(errno));

		exit(EXIT_FALSE);
	    }
       
	serv = getservbyport (htons(server_port), "tcp");
	if (serv != NULL)
	    address.sin_port = serv->s_port;
	else
	    {
		fprintf(stderr, _STRING_RED_": client_init() -> "
			"getservbyport()\n", strerror(errno));
		printf ("Server not running on this port or this port is not "
			"in /etc/services\n"
			"Please add \"bcount\t%d/tcp\" in "
			"/etc/services\n\tor \"doom\t%d/tcp\" "
			"in /etc/services\n", server_port, server_port);

		exit(EXIT_FALSE);
	    }

	return(EXIT_SUCCESS); 
}


int connect_to_server(void)
{
	csd = inet_socket_stream();

	if (connect (csd, (struct sockaddr *) &address, sizeof(address)) < 0)
	    {
		close_ncurses_screen();

		fprintf(stderr, _STRING_RED_": connect_to_server() -> "
			"connect()\n", strerror(errno));
		if (errno == ECONNREFUSED)
		    printf ("Connection refused or server not "
			    "running on this port.\n");

		if (errno == ECONNRESET)
		    printf ("Connection reset. Connection reset through "
			    "other proces.\n");

		if (errno == ECONNABORTED)
		    printf ("Connection aborted. I don't known why!?\n");
		    
		close(csd);

		exit(EXIT_FALSE);
	    }

	/* Read Server Welcome message */
	read_welcome_from_server();

	/* Send request and other data to server */
    	send_request_to_server(GET_METHOD, server_interface, BCNT_PROTOCOL_DEVEL, 
				data_type, message);
	
	/* Get header and body from server */
	get_header_and_body_from_server();
	
	close(csd);

	return(EXIT_SUCCESS);
}


int send_request_to_server(char *method, char *interface, char *protocol,
			    char *data_type, char *message)
{
	clear_buffer(transmit_buffer);

	if (!strcmp (message, ANULL))
	    {
		snprintf (transmit_buffer, sizeof(transmit_buffer), 
			    "%s /%s %s"CRLF
			    USER_AGENT": %s / Version %s"CRLF
			    DATA_TYPE": %s"CRLF
			    CRLF CRLF,
			    method, interface, protocol, CNAME, 
			    CVERSION, data_type);
	    }
	else
	    {
		snprintf (transmit_buffer, sizeof(transmit_buffer), 
			    "%s /%s %s"CRLF
			    USER_AGENT": %s / Version %s"CRLF
			    DATA_TYPE": %s"CRLF
			    MESSAGE": %s"CRLF
			    CRLF CRLF,
			    method, interface, protocol, CNAME, 
			    CVERSION, data_type, message);
	    }

	write (csd, transmit_buffer, strlen(transmit_buffer));

    	return(EXIT_SUCCESS);
}


int read_welcome_from_server(void)
{
	clear_buffer(received_buffer);
	bytes_read = 0;	

	/* Read Server Welcome message */
	while (strstr(received_buffer, HAPPY_FACE) == NULL)
	    bytes_read = read (csd, received_buffer, 
				sizeof(received_buffer) - 1);

	if (bytes_read > 0)
	    {
		/* add NULL byte */
		received_buffer[bytes_read] = '\0';

		sscanf (received_buffer, "%*s %s", server_name);
	    }
	else if (bytes_read < 0)
	    {
		close_ncurses_screen();
		fprintf(stderr, _STRING_RED_": read_welcome_from_server() ->"
			" read()\n", strerror(errno));
		close(csd);

		exit(EXIT_FALSE);
	    }

	return(EXIT_SUCCESS);
}


int get_header_and_body_from_server(void)
{
	clear_buffer(received_buffer);
	bytes_read = 0;	
	
	/* Read Server Header answer */
	while (strstr(received_buffer, END_BCNT_HEADER) == NULL)
	    bytes_read = read (csd, received_buffer, 
				sizeof(received_buffer) - 1);

	if (bytes_read > 0)
	    {
		sscanf (received_buffer, "%s %s %s", bcnt_server_protocol, 
			server_interface, server_answer);

		if (!strcmp (bcnt_server_protocol, BCNT_PROTOCOL_DEVEL) &&
		    !strcmp (bcnt_server_protocol, BCNT_PROTOCOL_1_0))
			protocol_error();

		if (!strcmp (server_answer, IF_NOT_FOUND))
		    interface_error();
		else if (!strcmp (server_answer, BAD_METHOD))
		    method_error();
		else if (!strcmp (server_answer, NOT_FOUND))
		    request_error();

		get_additional_request (received_buffer, SERVER_NAME);
		get_additional_request (received_buffer, SERVER_VERSION);

		if (!strncmp (server_version, "0.8pre5", 7))
		    server_version_error();
		else if (!strncmp (server_version, "0.7", 4))
		    server_version_error();
		else if (!strncmp (server_version, "0.6", 4))
		    server_version_error();
		else if (!strncmp (server_version, "0.5", 3))
		    server_version_error();
		else if (!strncmp (server_version, "0.2", 3))
		    server_version_error();
		else if (!strncmp (server_version, "0.1", 3))
		    server_version_error();

		get_additional_request (received_buffer, INTERFACE_IP);
		get_additional_request (received_buffer, CONTENT_LENGTH);

		/* If `END_BCNT_BODY' not dedected then read Server body */
		while (strstr(received_buffer, END_BCNT_BODY) == NULL)
		    bytes_read = read (csd, received_buffer, 
					sizeof(received_buffer) - 1);

		/* add NULL byte */
		received_buffer[bytes_read] = '\0';

		if (!strcmp (server_answer, STRING_OK))
		    {
			get_additional_request (received_buffer, BYTES_TX);
			get_additional_request (received_buffer, PACKETS_TX);
			get_additional_request (received_buffer, ERRORS_TX);
			get_additional_request (received_buffer, BYTES_RX);
			get_additional_request (received_buffer, PACKETS_RX);
			get_additional_request (received_buffer, ERRORS_RX);
			get_additional_request (received_buffer, MESSAGE);
		    }

	    }
	else if (bytes_read < 0)
	    {
		close_ncurses_screen();
		fprintf(stderr, _STRING_RED_": get_header_and_body_from_server()"
			" -> read()\n",	strerror(errno));
		close(csd);

		exit(EXIT_FALSE);
	    }

	return(EXIT_SUCCESS);
}


/* NIN */


