/*
   *
   * bcount.c - Bytes Counter
   *
   * This program makes statistics transferred and received
   * data over the network interfaces (TX / RX).
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * Depend:          Curses (aka Ncurses)
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include "bcount.h"
#include "activity_leds.h"
#include "signals.h"
#include "windows_utils.h"
#include "kernel_release.h"
#include "rotate_dash.h"
#include "fade_screen.h"
#include "license.h"




int main(int argc, char **argv, char **envp)
{
	int c = 0;
	fd_set readfd;
	struct timeval timeout;

	tvn.tv_sec = 0;
	tvn.tv_nsec = 50000;

	/* ! This is for bug report ! */
	strcpy (running_function, "main()");
	
	/* Set to default `locale' */
	setlocale (LC_ALL, "");

	/* Signals */
	signals_functions();

	/* Set default terminal as "linux" -> TERM=linux */
	**envp = set_terminal_type(TERM_LINUX);

	exec_name = strrchr (argv[0], '/');
	if (!exec_name)
	    exec_name = argv[0];
	else
	    ++exec_name;

	
/*	if (getuid() == 0 || geteuid() == 0)
	    {
		fprintf (stdout, "Run this as root? [y/n]: ");
		switch (c = getchar())
		    {
			case 'y':
			case 'Y':
			    fprintf (stdout, "\n");
			    break;
			case 'n':
			case 'N':
			    fprintf (stdout, "\n");
			    exit(EXIT_FALSE);
			    break;
			default:
			    fprintf (stdout, 
				    "Pleace choice 'y' or 'n' char.\n");
			    exit(EXIT_FALSE);
			    break;
		    }
	    }*/
	

	/* ****************************************************************** */
	/* IMPORTANT: This filling interface struct DON'T REMOVE THIS line! 
	    is->interfaces_names 
	    is->interfaces_counter
	    
	   and fill interface_info `Interface' struct.
	*/
	is = (struct interfaces_in_system *) 
	    xmalloc (sizeof (struct interfaces_in_system));

	get_interfaces_in_system();

	/* If default interface not found then set to Loopback */
	if (search_for_interface(default_interface_name) == INTERFACE_NOT_FOUND)
	    set_default_interface_as_loopback();
	
	/* Get default interface info ... like ip, netmask ...etc. */
	i = (struct interface_info *) 
	    xmalloc (sizeof (struct interface_info));

	get_interface_data(default_interface_name);

	/* Get local host name */
	strcpy (local_host_name, get_local_host_name());
	strcpy (smtp_server_name, local_host_name);
	/* ****************************************************************** */


	while ((c = getopt_long(argc, argv, "i:f:m:n:d:oplwrtsvVhg", 
		longopts, 0)) != EOF)
    	    switch (c)
        	{
		    case 'i':
			free_interface_info_struct();	
			i = (struct interface_info *) 
			    xmalloc (sizeof (struct interface_info));
			get_interface_data(optarg);
			break;
		    case 'f':
			log_file = optarg;
			break;
		    case 'm':
			remote_smtp_server = optarg;
			break;
		    case 'n':
			smtp_debug_file = optarg;
			smtp_session_debug_flag = TRUE;
			break;
		    case 'o':
			activity_leds_flag = FALSE;
			break;
		    case 'p':
			smtp_session_debug_flag = TRUE;
			break;
		    case 'd':
			data_type = optarg;
			check_data_type();
			break;
		    case 'w':
			write_log();
			break;
		    case 'r':
			read_log();
			break;
		    case 't':
			write_log_as_text();
			break;
		    case 's':
			summary_bytes();
			break;
		    case 'v':
		    case 'V':
			print_version();
			puts("");
			exit_prog(EXIT_SUCCESS);
		    case 'l':
			print_license();
			break;
		    case 'h':
			print_usage();
			break;
		    case 'g':
			signals_information();
			break;
		    default:
			print_info();
			break;
		}
        c = 0 ;

	/* Print version of bcount when we execute */
	print_welcome();
	
	/* Check if we don't have old kernel */
	kernel_release();

	/* Check if we don't have old log file */
	check_if_old_log();

	/* Get info about log file */
	get_file_info(log_file);

	/* Initializing SMTP Client */
	smtp_client_init();   
   
	/* Initialize Curses */
	init_ncurses_screen();

//reset:   
   
	/* Drawing Windows, panels, etc. */
	draw_ncurses_screen();


	/* Fire all threads up */
	if (pthreads_starting_up() == EXIT_FALSE)
	    {
		if (curses_flag == TRUE)
		    close_ncurses_screen();
		    
		return(EXIT_FALSE);
	    }
	

	do_top:
	c = 0;
	
	do
	    {
		FD_ZERO (&readfd);
		FD_SET (0, &readfd);
		timeout.tv_sec = 0;
		/* FIXME: 500000usec = 0.5sec; 1000000usec = 1sec */
		/* timeout.tv_usec = 100000;*/ /* 0.1sec */
		timeout.tv_usec = 50000; /* 0.05sec */

		switch ((select (1, &readfd, NULL, NULL, &timeout)) > 0)
		    {
			case 0:
			    if ((FD_ISSET (0, &readfd)) == 0)
				{
				    pthread_mutex_lock(&mutex_id);
				    move(2, 36);
				    rotate_dash(now_window);
				    pthread_mutex_unlock(&mutex_id);
				}
			    break;
			case -1:
			    if (errno != EINTR)
				{
				    fprintf (stderr, _STRING_RED_": main() -> "
					    "select()\n", strerror(errno));

				    exit_prog(EXIT_FALSE);
				}
			    else
				continue;
			    break;
			case 1:
			case 2:
			    if (FD_ISSET (0, &readfd))
				{
				    pthread_mutex_lock(&mutex_id);
				    remove_dash(now_window);
				    pthread_mutex_unlock(&mutex_id);

				    switch (c = getch())
					{
					    case KEY_F(1):
					    case 'a':
					    case 'A':
						pthread_mutex_lock(&mutex_id);
						make_ncurses_about_window();
						pthread_mutex_unlock(&mutex_id);
						break;
					    case 'c':
					    case 'C':
						pthread_mutex_lock(&mutex_id);
						send_mail_clear_status();
						pthread_mutex_unlock(&mutex_id);
						break;
					    case KEY_F(3):
					    case 'd':
					    case 'D':
						pthread_mutex_lock(&mutex_id);
						make_ncurses_devices_window();
						pthread_mutex_unlock(&mutex_id);
						break;
					    case 'i':
					    case 'I':
						choice_interface();
						break;
					    case KEY_F(4):
					    case 'e':
					    case 'E':
						pthread_mutex_lock(&mutex_id);
						make_ncurses_interface_info_window();
						pthread_mutex_unlock(&mutex_id);
						break;
					    case 'm':
					    case 'M':
						make_mail();
                            			break;
                    			    case 's':
                			    case 'S':
                    				searching();
                				break;
                    			    case 'l':
                			    case 'L':
						if (activity_leds_flag == TRUE)
						    activity_leds_flag = FALSE;
						else if (activity_leds_flag == FALSE)
						    activity_leds_flag = TRUE;
						if (active_window_info_flag == TRUE)
						    {
							pthread_mutex_lock(&mutex_id);
							make_ncurses_info_window();
							pthread_mutex_unlock(&mutex_id);
						    }
                				break;
                    			    case 'r':
                			    case 'R':
						pthread_mutex_lock(&mutex_id);
						draw_ncurses_screen();
						pthread_mutex_unlock(&mutex_id);
                				break;
					    case KEY_F(5):
                    			    case 'o':
                			    case 'O':
						pthread_mutex_lock(&mutex_id);
						make_ncurses_other_data_window();
						pthread_mutex_unlock(&mutex_id);
                				break;
					    case KEY_F(2):
                    			    case 'h':
                			    case 'H':
						pthread_mutex_lock(&mutex_id);
						make_ncurses_info_window();
						pthread_mutex_unlock(&mutex_id);
                				break;
                    			    case 'x':
                			    case 'X':
						pthread_mutex_lock(&mutex_id);
						make_ncurses_down_window();
						pthread_mutex_unlock(&mutex_id);
						shutdown_system(SYS_REBOOT, 9);
                				break;
				        // case '\033':goto reset ;
					           
					    default:
						break;

					}

				}
			    break;

		    }

	    }
	while (tolower(c) != 'q');

	quit_confirmation();
	goto do_top;

	return(EXIT_SUCCESS);
}


/* Initialize Windows ;-) */
void init_ncurses_screen(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "init_screen()");

	main_window = initscr();
	getmaxyx (stdscr, y_max, x_max);
	void_window = derwin (stdscr, y_max, x_max, 0, 0);
	interface_window = derwin (stdscr, 1, x_max - 12, 0, 1);
	now_window = derwin (stdscr, y_max - 12, x_max - 45, 1, 2);
	getmaxyx (now_window, now_y_max, now_x_max);
	total_window = derwin (stdscr, y_max - 19, x_max - 45, 14, 2);
	search_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	info_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	about_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	devices_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	interface_info_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	other_data_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	mail_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	down_window = derwin (stdscr, y_max - 14, x_max - 40, 1, 38);
	status_window = derwin (stdscr, 0, x_max - 10, y_max - 2, 1);
	mail_status_window = derwin (stdscr, 0, x_max - 72, y_max - 2, 71);
	time_window = derwin (stdscr, 0, x_max - 70, 0, 69);
	speed_window = derwin (stdscr, y_max - 20, x_max - 40, 12, 38);
	quit_window = derwin (stdscr, y_max - 21, x_max - 40, 10, 20);
	quit_window_sh = derwin (stdscr, y_max - 21, x_max - 40, 11, 22);
	sys_info_window = derwin (stdscr, y_max - 22, x_max - 4, 20, 2);
	choice_if_window = derwin (stdscr, y_max - 22, x_max - 40, 17, 38);

	cbreak();
	noecho();

	keypad(stdscr, KEYPAD_ON);
		
	/* Initialize Curses Colours */
	init_ncurses_screen_color();
}


/* Shutdown threads */
void syncing_and_down_threads(void)
{
	/* Don't remove this! */
    	pthreads_shutting_down(REFRESH_THREAD);

	make_ncurses_threads_sync_window();

	/* ...syncing threads */
	/* Killing stats thread ...syncing */
	wattroff (quit_window, A_UNDERLINE);
	mvwaddstr (quit_window, 1, 2, "STATS...");
	wrefresh (quit_window);
	
	pthreads_shutting_down(STATS_THREAD);
	
	wattron (quit_window, A_UNDERLINE);
	mvwaddstr (quit_window, 1, 11, "done :-)");
	wrefresh (quit_window);
	
	/* Killing interface speed thread ...syncing */
	wattroff (quit_window, A_UNDERLINE);
	mvwaddstr (quit_window, 1, 21, "SPEED...");
    	wrefresh (quit_window);
    	
	pthreads_shutting_down(SPEED_THREAD);
    	
	wattron (quit_window, A_UNDERLINE);
    	mvwaddstr (quit_window, 1, 30, "done :-)");
    	wrefresh (quit_window);

	/* Killing refresh stats thread ...syncing */
    	wattroff (quit_window, A_UNDERLINE);
    	mvwaddstr (quit_window, 2, 2, "REFRESH...");
    	wrefresh (quit_window);
        
	pthreads_shutting_down(REFRESH_THREAD);
    	sleep(SLEEP_TIME);

    	wattron (quit_window, A_UNDERLINE);
    	mvwaddstr (quit_window, 2, 12, " done :-)");
    	wrefresh (quit_window);

        sleep(SLEEP_TIME);

	wattroff (quit_window, A_UNDERLINE);
    	mvwaddstr (quit_window, 2, 24, "<Synced Ok>");
    	wrefresh (quit_window);

        sleep(SLEEP_TIME);

        wattroff (quit_window, A_UNDERLINE);
        wrefresh (quit_window);

	pthreads_shutting_down(ALL_THREADS);
}


void make_ncurses_interface_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_interface_window()");

	wattron (interface_window, COLOR_PAIR(2));
	wattroff (interface_window, A_UNDERLINE);
	fill_panel (interface_window);
	mvwaddstr (interface_window, 0, 1, "-");
	mvwaddstr (interface_window, 0, 3, "Interface:");
	mvwaddstr (interface_window, 0, 23, "IP:");
	mvwaddstr (interface_window, 0, 43, "Encap:");
	wattron (interface_window, COLOR_PAIR(4));
	mvwprintw (interface_window, 0, 14, "%s", i->name);
	mvwprintw (interface_window, 0, 27, "%s", i->ip_addr);
	mvwprintw (interface_window, 0, 50, "%s", i->description);

	wrefresh (interface_window);
}


void make_ncurses_now_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_now_window()");

	wattron (now_window, COLOR_PAIR(1));
	wattroff (now_window, A_UNDERLINE);
	box (now_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (now_window, "| Now TX/RX: |", TOP);
	fill_window (now_window);
	mvwaddstr (now_window, 1, 2, "Current speed TX");
	mvwaddstr (now_window, 2, 2, "Transmitted Data");
	mvwaddstr (now_window, 3, 5, "Packets speed");
	mvwaddstr (now_window, 4, 11, "Packets");
	mvwaddstr (now_window, 5, 12, "Errors");
	mvwaddstr (now_window, 6, 2, "Current speed RX");
	mvwaddstr (now_window, 7, 5, "Received Data");
	mvwaddstr (now_window, 8, 5, "Packets speed");
	mvwaddstr (now_window, 9, 11, "Packets");
	mvwaddstr (now_window, 10, 12, "Errors");
	mvwaddstr (now_window, 11, 2, "[Act. led:   ]   [Act. led:   ]");
	wattron (now_window, COLOR_PAIR(11));
	mvwaddstr (now_window, 11, 13, "TX");
	mvwaddch (now_window, 1, 18, ':');
	mvwaddch (now_window, 2, 18, ':');
	mvwaddch (now_window, 3, 18, ':');
	mvwaddch (now_window, 4, 18, ':');
	mvwaddch (now_window, 5, 18, ':');
	wattron (now_window, COLOR_PAIR(12));
	mvwaddstr (now_window, 11, 30, "RX");
	mvwaddch (now_window, 6, 18, ':');
	mvwaddch (now_window, 7, 18, ':');
	mvwaddch (now_window, 8, 18, ':');
	mvwaddch (now_window, 9, 18, ':');
	mvwaddch (now_window, 10, 18, ':');
	wattron (now_window, COLOR_PAIR(1));

	wrefresh (now_window);
}


void make_ncurses_devices_window(void)
{
	int ic = 0, flags = 0;

	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_devices_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = TRUE;
	
	/* ****************************************************************** */
	/* IMPORTANT: This filling interface struct DON'T REMOVE THIS line! 
	    is->interfaces_names 
	    is->interfaces_counter
	*/
	free_interfaces_in_system_struct();

	is = (struct interfaces_in_system *) 
	    xmalloc (sizeof (struct interfaces_in_system));

	get_interfaces_in_system();
	/* ****************************************************************** */

	wattron (devices_window, COLOR_PAIR(1));
	wattroff (devices_window, A_UNDERLINE);
	box (devices_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (devices_window, "| Interfaces in system: |", TOP);
	fill_window (devices_window);
	wattron (devices_window, COLOR_PAIR(2));
	text_window_in_middle (devices_window, " - UP -> ife is up; DN -> ife is down ", BOTTOM);
	wattron (devices_window, COLOR_PAIR(1));
	
	while (ic < is->interfaces_counter)
	    {
		wattroff (devices_window, A_UNDERLINE);
		mvwaddstr (devices_window, ic + 1, 2, "-> <  > [ ]");
	    	mvwprintw (devices_window, ic + 1, 21, "[%s]", 
			    get_interface_desc(is->interfaces_names[ic]));

		flags = get_interface_flags(is->interfaces_names[ic]);

		if (flags & IFF_UP)
	    	    {
			wattron (devices_window, COLOR_PAIR(11));
			mvwprintw (devices_window, ic + 1, 6, "%s", UP_STRING);
		    }
		else
		    {
		    	wattron (devices_window, COLOR_PAIR(12));
	    		mvwprintw (devices_window, ic + 1, 6, "%s", DOWN_STRING);
		    }

		wattron (devices_window, COLOR_PAIR(1));
		wattron (devices_window, A_UNDERLINE);
	    	mvwprintw (devices_window, ic + 1, 11, "%d", ic);
	    	mvwprintw (devices_window, ic + 1, 14, "%s", 
			    is->interfaces_names[ic]);
		
		ic++;
	    }

	ic = 0;
	    
	wrefresh (devices_window);
}


void make_ncurses_about_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_about_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = FALSE;

	wattron (about_window, COLOR_PAIR(1));
	wattroff (about_window, A_UNDERLINE);
	box (about_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (about_window, "| About this: |", TOP);
	fill_window (about_window);
	mvwaddstr (about_window, 1, 4, "BYTES COUNTER v    stable :-)");
	mvwaddstr (about_window, 3, 4, "This stuff display detailed");
	mvwaddstr (about_window, 4, 4, "information about the number");
	mvwaddstr (about_window, 5, 4, "of bytes and packets that have");
	mvwaddstr (about_window, 6, 4, "been transmitted and received");
	mvwaddstr (about_window, 7, 4, "over a network interface.");
	mvwaddstr (about_window, 9, 4, "Enjoy!");

	wattron (about_window, A_UNDERLINE);
	mvwprintw (about_window, 1, 19, "%s", VERSION);
	wattroff (about_window, A_UNDERLINE);

	wrefresh (about_window);
}


void make_ncurses_info_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_info_window()");

	active_window_info_flag = TRUE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = FALSE;

	wattron (info_window, COLOR_PAIR(1));
	wattroff (info_window, A_UNDERLINE);
	box (info_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (info_window, "| Info (available keys): |", TOP);
	fill_window (info_window);
	mvwaddstr (info_window, 1, 3, "'F1; a' - about this program");
	mvwaddstr (info_window, 2, 3, "    'c' - clear mail status");
	mvwaddstr (info_window, 3, 3, "'F3; d' - available interfaces");
	mvwaddstr (info_window, 4, 3, "'F4; e' - info about interface");
	mvwaddstr (info_window, 5, 3, "    'i' - choice interface");
	mvwaddstr (info_window, 6, 3, "    'l' - turn leds on/off");
	if (activity_leds_flag == TRUE)
	    mvwaddstr (info_window, 6, 30, "[on] ");
	else
	    mvwaddstr (info_window, 6, 30, "[off]");
	mvwaddstr (info_window, 7, 3, "    'm' - send e-mail via SMTP");
	mvwaddstr (info_window, 8, 3, "'F5; o' - other TX/RX data");
	mvwaddstr (info_window, 9, 3, "    's' - search in log file");

	wrefresh (info_window);
}


void make_ncurses_interface_info_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_interface_info_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = TRUE;
	active_window_devices_flag = FALSE;

	wattron (interface_info_window, COLOR_PAIR(1));
	wattroff (interface_info_window, A_UNDERLINE);
	box (interface_info_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (interface_info_window, "| Interface setup info: |", TOP);
	fill_window (interface_info_window);
	mvwaddstr (interface_info_window, 1, 3, "Driver:                      ");
	mvwaddstr (interface_info_window, 2, 3, "Netmask:                     ");
	mvwaddstr (interface_info_window, 3, 3, "Broadcast:                   ");
	mvwaddstr (interface_info_window, 4, 3, "PtoP IP address:                ");
	mvwaddstr (interface_info_window, 5, 3, "HW address:                  ");
	mvwaddstr (interface_info_window, 6, 3, "Metric:    Dma:    Irq:      ");
	mvwaddstr (interface_info_window, 7, 3, "Mtu:       Base addr:        ");
	mvwaddstr (interface_info_window, 8, 3, "IP class:    PtoP IP class:  ");
	mvwaddstr (interface_info_window, 10, 2, "<                                  >");
	mvwaddstr (interface_info_window, 10, 3, " Link:      Speed:                ");
	wattron (interface_info_window, A_UNDERLINE);
	mvwprintw (interface_info_window, 1, 11, "%s [%s]", i->eth.driver, 
		    i->eth.bus_info);
	mvwprintw (interface_info_window, 2, 12, "%s", i->netmask);
	mvwprintw (interface_info_window, 3, 14, "%s", i->broadcast);
	mvwprintw (interface_info_window, 4, 20, "%s", i->ip_daddr);
	mvwprintw (interface_info_window, 5, 15, "%s", i->hw_addr);
	mvwprintw (interface_info_window, 6, 11, "%d", i->metric);
	mvwprintw (interface_info_window, 6, 19, "%d", i->dma);
	mvwprintw (interface_info_window, 6, 27, "%d", i->irq);
	mvwprintw (interface_info_window, 7, 8, "%d", i->mtu);

	if (i->base_addr != 0)
	    mvwprintw (interface_info_window, 7, 25, "0x%x", i->base_addr);
	else
	    mvwprintw (interface_info_window, 7, 25, "%s", not_available);

	mvwprintw (interface_info_window, 8, 13, "%c", i->ip_addr_class);

	if (i->ip_daddr_class == '?')
	    mvwprintw (interface_info_window, 8, 31, "%s", not_available);
	else
	    mvwprintw (interface_info_window, 8, 31, "%c", i->ip_daddr_class);

	if (strlen(i->nflags) <= 35)
	    mvwprintw (interface_info_window, 9, 3, "%s", i->nflags);
	else
	    mvwaddstr (interface_info_window, 9, 3, "TOO MANY FLAGS TO FIT IN WINDOW!");

	mvwprintw (interface_info_window, 10, 22, "%s, %s", 
		    i->eth.speed, i->eth.duplex);

	if (!strcmp(i->eth.link, STRING_YES))
	    {
		wattron (interface_info_window, COLOR_PAIR(11));
		wattroff (interface_info_window, A_UNDERLINE);
		mvwprintw (interface_info_window, 10, 10, "%s", i->eth.link);
	    }
	else if (!strcmp(i->eth.link, STRING_NO))
	    {
		wattron (interface_info_window, COLOR_PAIR(12));
		wattroff (interface_info_window, A_UNDERLINE);
		mvwprintw (interface_info_window, 10, 10, "%s", i->eth.link);
	    }
	else
	    mvwprintw (interface_info_window, 10, 10, "%s", i->eth.link);

	wattroff (interface_info_window, A_UNDERLINE);
	wrefresh (interface_info_window);
}


void make_ncurses_other_data_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_other_data_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = TRUE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = FALSE;

	wattron (other_data_window, COLOR_PAIR(1));
	wattroff (other_data_window, A_UNDERLINE);
	box (other_data_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (other_data_window, "| Other Data TX/RX: |", TOP);
	fill_window (other_data_window);
	mvwaddstr (other_data_window, 2, 3, "- Transmitted -");
	mvwaddstr (other_data_window, 3, 5, "Carrier:      ");
	mvwaddstr (other_data_window, 4, 7, "Colls:      ");
	mvwaddstr (other_data_window, 5, 7, "Compr:      ");
	mvwaddstr (other_data_window, 6, 8, "Drop:      ");
	mvwaddstr (other_data_window, 7, 8, "Fifo:      ");
	mvwaddstr (other_data_window, 2, 21, "- Received -");
	mvwaddstr (other_data_window, 3, 23, "Frame:      ");
	mvwaddstr (other_data_window, 4, 23, "Multi:      ");
	mvwaddstr (other_data_window, 5, 23, "Compr:      ");
	mvwaddstr (other_data_window, 6, 24, "Drop:      ");
	mvwaddstr (other_data_window, 7, 24, "Fifo:      ");

	wrefresh (other_data_window);
}


void make_ncurses_void_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_void_window()");

	wattron (void_window, COLOR_PAIR(1));
	wattron (void_window, A_NORMAL);
	wattroff (void_window, A_UNDERLINE);
	box (void_window, ACS_VLINE, ACS_HLINE);
	fill_window (void_window);

	mvwaddch (void_window, y_max - 1, 3, ACS_VLINE);
	mvwaddch (void_window, y_max - 1, x_max - 56, ACS_VLINE);
	mvwaddstr (void_window, y_max - 1, 4, " Press 'q' for exit ");

	mvwaddch (void_window, y_max - 1, x_max - 4, ACS_VLINE);
	mvwaddch (void_window, y_max - 1, x_max - 25, ACS_VLINE);
	mvwaddstr (void_window, y_max - 1, x_max - 24, " Press 'h' for info ");

	wrefresh (void_window);
}


void make_ncurses_total_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_total_window()");

	wattron (total_window, COLOR_PAIR(1));
	wattron (total_window, A_NORMAL);
	wattroff (total_window, A_UNDERLINE);
	box (total_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (total_window, 
				"| Total TX/RX: (in log file) |", TOP);
	fill_window (total_window);
	mvwaddstr (total_window, 1, 2, "F:             L:");
	mvwaddstr (total_window, 2, 2, "Transmitted Data:");
	mvwaddstr (total_window, 3, 5, "Received Data:");
	mvwaddstr (total_window, 4, 11, "Records:");

	wrefresh (total_window);
}


void make_ncurses_speed_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_speed_window()");

	wattron (speed_window, COLOR_PAIR(1));
	wattroff (speed_window, A_UNDERLINE);
	box (speed_window, ACS_VLINE, ACS_HLINE);
	fill_window (speed_window);
	text_window_in_middle (speed_window, "| Speed on an interface: |", TOP);
	mvwaddstr (speed_window, 1, 5, "Current Speed");
	mvwaddstr (speed_window, 1, 23, "Maximal Speed");
	mvwaddstr (speed_window, 2, 2, "<C:              ");
	mvwaddch (speed_window, 2, 19, ACS_VLINE);
	mvwaddch (speed_window, 3, 19, ACS_VLINE);
	mvwaddstr (speed_window, 2, 21, "M:              >");
	wattron (speed_window, COLOR_PAIR(9));
	mvwaddstr (speed_window, 2, 24, "0.00 KB/s");
	mvwaddstr (speed_window, 3, 24, "0 P/s");

	wrefresh (speed_window);
}


void make_ncurses_shutdown_window(char *reboot_or_halt, int wait)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_shutdown_window()");

	wattron (quit_window_sh, COLOR_PAIR(7));
	wbkgd (quit_window_sh, ' ');
	wattron (quit_window, COLOR_PAIR(10));
	wattroff (quit_window, A_UNDERLINE);
	box (quit_window, ACS_VLINE, ACS_HLINE);
	fill_window (quit_window);
	text_window_in_middle (quit_window, "| Shutdown system . . . |", TOP);
	mvwaddstr (quit_window, 1, 3, "The system is going down until ...");
	mvwaddstr (quit_window, 2, 20, "sec.");
	if (!strcmp (reboot_or_halt, SYS_REBOOT))
	    text_window_in_middle (quit_window, " ... [Reboot] in progrss ... ", BOTTOM);
	else
	    text_window_in_middle (quit_window, " ... [Halt] in progrss ... ", BOTTOM);

	wrefresh (quit_window_sh);
	wrefresh (quit_window);

	wattron (quit_window, A_UNDERLINE);
	while (wait != 0)
	    {
		terminal_beep();

    		mvwprintw (quit_window, 2, 18, "%d", wait);
		wrefresh (quit_window);

		sleep(SLEEP_TIME);
		
		wait--;
	    }

	terminal_beep();

	mvwprintw (quit_window, 2, 18, "%d", wait);
	wrefresh (quit_window);

	sleep(SLEEP_TIME);
}


void make_ncurses_threads_sync_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_threads_sync_window()");

	wattron (quit_window_sh, COLOR_PAIR(7));
	wbkgd (quit_window_sh, ' ');
	wattron (quit_window, COLOR_PAIR(8));
	wattroff (quit_window, A_UNDERLINE);
	box (quit_window, ACS_VLINE, ACS_HLINE);
	fill_window (quit_window);

	text_window_in_middle (quit_window, "| Syncing pTHREAD's: |", TOP);
	wattron (quit_window, A_BLINK | COLOR_PAIR(8));
	text_window_in_middle (quit_window, " Press Ctrl-C if fail! ", BOTTOM);
	wattroff (quit_window, A_BLINK | COLOR_PAIR(8));
	wattron (quit_window, COLOR_PAIR(8));

	wrefresh (quit_window_sh);
	wrefresh (quit_window);
}


void make_ncurses_quit_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_quit_window()");

	wattron (quit_window_sh, COLOR_PAIR(7));
	wbkgd (quit_window_sh, ' ');
	wattron (quit_window, COLOR_PAIR(6));
	wattroff (quit_window, A_UNDERLINE);
	box (quit_window, ACS_VLINE, ACS_HLINE);
	fill_window (quit_window);

	text_window_in_middle (quit_window, 
				    "| Exit question ;-) |", TOP);
	mvwaddstr (quit_window, 1, 7, "Do you really want quit?...");
	mvwaddstr (quit_window, 2, 14, "[ ]es / [ ]o");
	wattron (quit_window, A_UNDERLINE);
	mvwaddstr (quit_window, 2, 15, "Y");
	mvwaddstr (quit_window, 2, 23, "N");
	wattroff (quit_window, A_UNDERLINE);

	wrefresh (quit_window_sh);
	wrefresh (quit_window);
}


void make_ncurses_sys_info_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_sys_info_window()");

	wattron (sys_info_window, COLOR_PAIR(1));
	wattroff (sys_info_window, A_UNDERLINE);
	box (sys_info_window, ACS_VLINE, ACS_HLINE);
	fill_window (sys_info_window);
	mvwprintw (sys_info_window, 0, 2, "[ CPU: ~%.1f MHz ]", get_cpu_clock_speed());
	mvwaddstr (sys_info_window, 0, 55, "[ MF:             ]");
	text_window_in_middle (sys_info_window, "| System info: |", TOP);
	mvwaddstr (sys_info_window, 1, 2, "< Uptime:                          >");
	mvwaddstr (sys_info_window, 1, 39, "< Host name:                      >");
	wattron (sys_info_window, A_UNDERLINE);

	mvwprintw (sys_info_window, 1, 52, "%s", local_host_name);
	wattroff (sys_info_window, A_UNDERLINE);

	wrefresh (sys_info_window);
}


void make_ncurses_choice_if_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_choice_if_window()");

	wattron (choice_if_window, COLOR_PAIR(1));
	wattroff (choice_if_window, A_UNDERLINE);
	box (choice_if_window, ACS_VLINE, ACS_HLINE);
	fill_window (choice_if_window);
	text_window_in_middle (choice_if_window, "| Choice interface: |", TOP);
	mvwaddstr (choice_if_window, 1, 2, "Interface:");
	mvwaddstr (choice_if_window, 1, 24, "<   available >");
	wattron (choice_if_window, A_UNDERLINE);

	/* Get number of available interfaces in system */
	mvwprintw (choice_if_window, 1, 26, "%d", get_interfaces_amount());

	wattroff (choice_if_window, A_UNDERLINE);
	wattron (choice_if_window, COLOR_PAIR(2));
	mvwaddstr (choice_if_window, 1, 13, "          ");

	wrefresh (choice_if_window);
}


void make_ncurses_status_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_status_window()");

	wattron (status_window, COLOR_PAIR(2));
	wattroff (status_window, A_UNDERLINE);
	fill_panel (status_window);
	mvwprintw (status_window, 0, 0, " - Log file: %s "
		    "(%d Bytes | UID: %d GID: %d) ", 
		    log_file, log_file_size, file_uid, file_gid);

	wrefresh (status_window);
}


void make_ncurses_mail_status_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_mail_status_window()");

	wattron (mail_status_window, COLOR_PAIR(2));
	wattroff (mail_status_window, A_UNDERLINE);
	fill_panel (mail_status_window);
	mvwaddch (mail_status_window, 0, 0, ACS_VLINE);

	wrefresh (mail_status_window);
}


void make_ncurses_time_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_time_window()");

	wattron (time_window, COLOR_PAIR(2));
	wattroff (time_window, A_UNDERLINE);
	fill_panel (time_window);
	mvwaddch (time_window, 0, 0, ACS_VLINE);

	wrefresh (time_window);
}


void make_ncurses_search_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_search_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = FALSE;

	wattron (search_window, COLOR_PAIR(1));
	wattroff (search_window, A_UNDERLINE);
	box (search_window, ACS_VLINE, ACS_HLINE);
	fill_window (search_window);
	text_window_in_middle (search_window, 
				"| Search TX/RX: (in log file) |", TOP);
	wattron (search_window, COLOR_PAIR(2));
	text_window_in_middle (search_window, 
			    "  - Press 's' for search in log file  ", BOTTOM);
	mvwaddstr (search_window, 2, 20, "    -  -  ");
	mvwaddstr (search_window, 4, 20, "    -  -  ");
	wattron (search_window, COLOR_PAIR(1));
	mvwaddstr (search_window, 6, 7, "Transmitted Data:");
	mvwaddstr (search_window, 7, 10, "Received Data:");
	mvwaddstr (search_window, 2, 9, "Date from:");
	mvwaddstr (search_window, 4, 11, "Date to:");
	wattron (search_window, COLOR_PAIR(3));
	mvwprintw (search_window, 3, 11, "Example: %s", get_localdate());
	wattron (search_window, COLOR_PAIR(1));

	wrefresh (search_window);
}


void make_ncurses_mail_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_mail_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = FALSE;

	wattron (mail_window, COLOR_PAIR(1));
	wattroff (mail_window, A_UNDERLINE);
	box (mail_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (mail_window, "| Check and mail: |", TOP);
	fill_window (mail_window);
	mvwaddstr (mail_window, 2, 2, "If transmitted [ ]");
	mvwaddstr (mail_window, 2, 25, "received [ ]");
	mvwaddch (mail_window, 2, 22, ACS_VLINE);
	mvwaddstr (mail_window, 4, 12, "MB then send e-mail to");
	mvwprintw (mail_window, 8, 3, "Sent ok: %d           Send error: %d",
		    email_sent_ok, email_send_error);
	wattron (mail_window, COLOR_PAIR(2));
	mvwaddstr (mail_window, 2, 18, " ");
	mvwaddstr (mail_window, 2, 35, " ");
	mvwaddstr (mail_window, 4, 5, "      ");
	mvwaddstr (mail_window, 6, 2, "                                    ");
	text_window_in_middle (mail_window, 
			    "  - `Enter' choice :: `Space' select  ", BOTTOM);

	wrefresh (mail_window);
}


void make_ncurses_down_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "make_ncurses_down_window()");

	active_window_info_flag = FALSE;
	active_window_other_data_flag = FALSE;
	active_window_interface_info_flag = FALSE;
	active_window_devices_flag = FALSE;

	wattron (down_window, COLOR_PAIR(1));
	wattroff (down_window, A_UNDERLINE);
	box (down_window, ACS_VLINE, ACS_HLINE);
	text_window_in_middle (down_window, "| Check and shutdown: |", TOP);
	fill_window (down_window);
	mvwaddstr (down_window, 2, 2, "If transmitted [ ]");
	mvwaddstr (down_window, 2, 25, "received [ ]");
	mvwaddch (down_window, 2, 22, ACS_VLINE);
	mvwaddstr (down_window, 4, 12, "MB then send e-mail to");
	wattron (down_window, COLOR_PAIR(2));
	mvwaddstr (down_window, 2, 18, " ");
	mvwaddstr (down_window, 2, 35, " ");
	mvwaddstr (down_window, 4, 5, "      ");
	text_window_in_middle (down_window, 
			    "  - `Enter' choice :: `Space' select  ", BOTTOM);

	wrefresh (down_window);
}


void refresh_other_data_window(void)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "refresh_other_data_window()");

	wattroff (other_data_window, A_UNDERLINE);
	mvwaddstr (other_data_window, 3, 5, "Carrier:      ");
	mvwaddstr (other_data_window, 4, 7, "Colls:      ");
	mvwaddstr (other_data_window, 5, 7, "Compr:      ");
	mvwaddstr (other_data_window, 6, 8, "Drop:      ");
	mvwaddstr (other_data_window, 7, 8, "Fifo:      ");
	mvwaddstr (other_data_window, 3, 23, "Frame:      ");
	mvwaddstr (other_data_window, 4, 23, "Multi:      ");
	mvwaddstr (other_data_window, 5, 23, "Compr:      ");
	mvwaddstr (other_data_window, 6, 24, "Drop:      ");
	mvwaddstr (other_data_window, 7, 24, "Fifo:      ");

	wattron (other_data_window, A_UNDERLINE);
	mvwprintw (other_data_window, 3, 14, "%lu", i->stats.tx_carrier);
	mvwprintw (other_data_window, 4, 14, "%lu", i->stats.tx_colls);
	mvwprintw (other_data_window, 5, 14, "%lu", i->stats.tx_compressed);
	mvwprintw (other_data_window, 6, 14, "%lu", i->stats.tx_drop);
	mvwprintw (other_data_window, 7, 14, "%lu", i->stats.tx_fifo);
	mvwprintw (other_data_window, 3, 30, "%lu", i->stats.rx_frame);
	mvwprintw (other_data_window, 4, 30, "%lu", i->stats.rx_multicast);
	mvwprintw (other_data_window, 5, 30, "%lu", i->stats.rx_compressed);
	mvwprintw (other_data_window, 6, 30, "%lu", i->stats.rx_drop);
	mvwprintw (other_data_window, 7, 30, "%lu", i->stats.rx_fifo);
	wattroff (other_data_window, A_UNDERLINE);

	wrefresh (other_data_window);
}


void refresh_memory_status(void)
{
	wattroff (sys_info_window, A_UNDERLINE);
	mvwaddstr (sys_info_window, 0, 61, "            ");
	mvwprintw (sys_info_window, 0, 61, "%d KB", get_free_memory());
	wrefresh (sys_info_window);
}


void refresh_rss_memory_status(void)
{
	/* Print rss memory */
	wattron (void_window, COLOR_PAIR(1));
	wattroff (void_window, A_UNDERLINE);
    	mvwaddstr (void_window, y_max -1, 32, "<              >");
	mvwprintw (void_window, y_max -1, 34, "PMU: %d KB", get_rss_memory(getpid()));
	wattroff (void_window, COLOR_PAIR(1));
	wrefresh (void_window);
}


void refresh_local_time_status(void)
{
	/* Print time in time_window */
	wattron (time_window, COLOR_PAIR(2));
	wattroff (time_window, A_UNDERLINE);
    	mvwaddstr (time_window, 0, 1, "       ");
	mvwprintw (time_window, 0, 1, "%s", get_localtime());
	wattroff (time_window, COLOR_PAIR(2));
	wrefresh (time_window);
}


void reset_speed_window(void)
{
	/* Reset top speed for choseing interface */
	wattron (speed_window, COLOR_PAIR(9));
	wattroff (speed_window, A_UNDERLINE);
	mvwaddstr (speed_window, 2, 24, "             ");
	mvwaddstr (speed_window, 2, 24, "0.00 KB/s");
	wrefresh (speed_window);
}


/* Window border, fill, text, etc. ... */
void draw_ncurses_screen(void)
{
	/* TRUE - Curses initialized; FALSE - Curses not initialized */
	curses_flag = TRUE;

	curs_set(INVISIBLE);

	/* Drawing WINDOW *void_window */
	make_ncurses_void_window();

	/* Drawing WINDOW *interface_window */
	make_ncurses_interface_window();
	/* Drawing WINDOW *time_window */
	make_ncurses_time_window();
	/* Drawing WINDOW *search_window */
	make_ncurses_search_window();
	/* Drawing WINDOW *now_window */
	make_ncurses_now_window();
	/* Drawing WINDOW *total_window */
	make_ncurses_total_window();
	/* Drawing WINDOW *speed_window */
	make_ncurses_speed_window();
	/* Drawing WINDOW *choice_if_window */
	make_ncurses_choice_if_window();
	/* Drawing WINDOW *sys_info_window */
	make_ncurses_sys_info_window();
	/* Drawing WINDOW *status_window */
	make_ncurses_status_window();
	/* Drawing WINDOW *mail_status_window */
	make_ncurses_mail_status_window();

	/* If you want disable fade in then please comment 
	    line below (fade_screen_in() function) */
	fade_screen_in();

	noecho();
	
	total_bytes();

	refresh();
}


/* Close initialized Windows */
void close_ncurses_screen(void)
{
	curses_flag = FALSE;

	erase();
	refresh();
	endwin();
}


/* I was take this from UConio. Sorry Pablo Picasso :-) */
void clrscr(void)
{
	fprintf(stdout, "\033[2J");
	fprintf(stdout, "\033[%i;%iH", 0, 0);
	fprintf(stdout, "\033[0;0;39m");
}


int get_system_info(void)
{
	FILE *proc_uptime;
	double uptime;
	const long minute = 60;
	const long hour = minute * 60;
	const long day = hour * 24;

	/* ! This is for bug report ! */
	strcpy (running_function, "get_system_info()");

	proc_uptime = fopen (PROC_UPTIME, "r");
	fscanf (proc_uptime, "%lf", &uptime);
	fclose (proc_uptime);
	
	wattron (sys_info_window, A_UNDERLINE);
	mvwprintw (sys_info_window, 1, 12, "%ld", (long) uptime / day);
	mvwprintw (sys_info_window, 1, 21, "%02ld  %02ld    %02ld   ",
		    ((long) uptime % day) / hour, 
		    ((long) uptime % hour) / minute,
		    (long) uptime % minute);
	wattroff (sys_info_window, A_UNDERLINE);
	if (uptime / day > 2 && uptime / day < 10)
		mvwaddstr (sys_info_window, 1, 14, "days");
	else if (uptime / day > 10)
		mvwaddstr (sys_info_window, 1, 15, "days"); 
	else if (uptime / day > 10 && uptime / day < 100)
		mvwaddstr (sys_info_window, 1, 15, "days");
	else if (uptime / day > 100)
		mvwaddstr (sys_info_window, 1, 16, "days");
	else
	    mvwaddstr (sys_info_window, 1, 14, "day");
		
	mvwaddstr (sys_info_window, 1, 23, "h:");
	mvwaddstr (sys_info_window, 1, 27, "min:");
	mvwaddstr (sys_info_window, 1, 33, "sec");

	wrefresh (sys_info_window);
	
	return(EXIT_SUCCESS);
}


/* Get file info like: size, uid and gid */
int get_file_info (char *filename)
{
	struct stat *file_info = (struct stat *) xmalloc (sizeof(struct stat));
	char *rval;
	
	/* ! This is for bug report ! */
	strcpy (running_function, "get_file_info()");
	
	rval = get_file_type(filename);
	if (rval != IS_REG && rval != IS_LNK)
	    file_type_error(filename);
	
	if (lstat (filename, file_info) == -1)
	    {
		fprintf (stderr, _STRING_RED_": get_file_info() -> lstat()\n", 
			strerror(errno));
		free(file_info);

		exit_prog(EXIT_FALSE);
	    }

	log_file_size = file_info->st_size;
	if (log_file_size == 0)
	    {
		free(file_info);
		return(EXIT_FALSE);
	    }
	file_uid = file_info->st_uid;
	file_gid = file_info->st_gid;

	free(file_info);
	
	return(EXIT_SUCCESS);
}


/* Refresh info in Now Window */
void* refresh_now_window(void* unused)
{
    while (shutdown_thread_refresh_stats_window == FALSE) {

	pthread_mutex_lock(&mutex_id);

	/* ! This is for bug report ! */
	strcpy (running_function, "refresh_now_window()");

	if (active_window_other_data_flag == TRUE)
	    refresh_other_data_window();
	    
	wattron (now_window, COLOR_PAIR(1));
	wattron (now_window, A_UNDERLINE);
	wrefresh (now_window);

	mvwaddstr (now_window, 1, 20, "              ");

	if (i->speed.kbs.speed_current_tx_kbytes > 0 && i->speed.kbs.speed_current_tx_kbytes < 1)
	    mvwprintw (now_window, 1, 20, "%d B/s", 
			(unsigned int)(i->speed.kbs.speed_current_tx_kbytes * 1024));
	else
	    mvwprintw (now_window, 1, 20, "%.2f KB/s", 
			i->speed.kbs.speed_current_tx_kbytes);

	mvwaddstr (now_window, 2, 20, "              ");

	if (!strcmp (data_type, BYTES))
	    mvwprintw (now_window, 2, 20, "%llu B", i->stats.tx_bytes);
	else if (!strcmp (data_type, KBYTES))
	    mvwprintw (now_window, 2, 20, "%.2f KB", i->stats.tx_kbytes);
	else if (!strcmp (data_type, MBYTES))
	    mvwprintw (now_window, 2, 20, "%.2f MB", i->stats.tx_mbytes);
	else
	    mvwprintw (now_window, 2, 20, "%.2f MB", i->stats.tx_mbytes);

	if (no_more_mail_flag == FALSE && transmitted_flag == TRUE)
	    {
		if (i->stats.tx_mbytes > atoi(get_mail_digit)
		    && smtp_init_flag == TRUE)
		    {
			if (strlen(get_mail_address) > 0 && 
			    strlen(get_mail_digit) > 0)
			    {
				pthread_mutex_unlock(&mutex_id);
				connect_to_smtp_server ("sent"); 	
				pthread_mutex_lock(&mutex_id);
			    }

		    }

		if (smtp_init_flag == FALSE)
		    smtp_not_found();

	    }		

	mvwaddstr (now_window, 3, 20, "              ");
	mvwaddstr (now_window, 4, 20, "              ");
	mvwaddstr (now_window, 5, 20, "              ");

	mvwprintw (now_window, 3, 20, "%llu P/s", 
		    i->speed.pss.speed_current_tx_packets);
	mvwprintw (now_window, 4, 20, "%llu", i->stats.tx_packets);
	mvwprintw (now_window, 5, 20, "%lu", i->stats.tx_errs);


	mvwaddstr (now_window, 6, 20, "              ");

	if (i->speed.kbs.speed_current_rx_kbytes > 0 && i->speed.kbs.speed_current_rx_kbytes < 1)
	    mvwprintw (now_window, 6, 20, "%d B/s", 
			(unsigned int)(i->speed.kbs.speed_current_rx_kbytes * 1024));
	else
	    mvwprintw (now_window, 6, 20, "%.2f KB/s", 
			i->speed.kbs.speed_current_rx_kbytes);

	mvwaddstr (now_window, 7, 20, "              ");

	if (!strcmp (data_type, BYTES))
	    mvwprintw (now_window, 7, 20, "%llu B", i->stats.rx_bytes);
	else if (!strcmp (data_type, KBYTES))
	    mvwprintw (now_window, 7, 20, "%.2f KB", i->stats.rx_kbytes);
	else if (!strcmp (data_type, MBYTES))
	    mvwprintw (now_window, 7, 20, "%.2f MB", i->stats.rx_mbytes);
	else
	    mvwprintw (now_window, 7, 20, "%.2f MB", i->stats.rx_mbytes);

	if (no_more_mail_flag == FALSE && received_flag == TRUE)
	    {
		if (i->stats.rx_mbytes > atoi(get_mail_digit) 
		    && smtp_init_flag == TRUE)
		    {
			if (strlen(get_mail_address) > 0 && 
			    strlen(get_mail_digit) > 0)
			    {
				pthread_mutex_unlock(&mutex_id);
				connect_to_smtp_server ("get");
				pthread_mutex_lock(&mutex_id);
			    } 	

		    }

		if (smtp_init_flag == FALSE)
		    smtp_not_found();

	    }

	mvwaddstr (now_window, 8, 20, "              ");
	mvwaddstr (now_window, 9, 20, "              ");
	mvwaddstr (now_window, 10, 20, "              ");

	mvwprintw (now_window, 8, 20, "%llu P/s", 
		    i->speed.pss.speed_current_rx_packets);
	mvwprintw (now_window, 9, 20, "%llu", i->stats.rx_packets);
	mvwprintw (now_window, 10, 20, "%lu", i->stats.rx_errs);

	wattroff (now_window, A_UNDERLINE);

	/* Increment counter to 20 */
	counter++;

	/* Because timeout.tv_usec = 50000 (0.05sec) then if we want 
	    average speed KB/1sec refresh must be after counter == 20 */
	if (counter == 20)
	    {
		/* Simple refreshing data after 1sec */
		refresh_rss_memory_status();
		refresh_memory_status();
		refresh_local_time_status();
		get_system_info();
		
		/* Refresh IP if we change interface parameters */
		/*strcpy (i->ip_addr, get_interface_ip(i->name));

		wattron (interface_window, COLOR_PAIR(4));
		mvwaddstr (interface_window, 0, 27, "                ");
		mvwprintw (interface_window, 0, 27, "%s", i->ip_addr);

		wrefresh (interface_window);*/
		/* --- end --- */

		if (active_window_interface_info_flag == TRUE) 
		    {
			/* ************************************************* */
			/* Get info about i->name per 1sec if 
			    active_window_interface_info_flag == TRUE. 
			    We can change settings when bcount is running! */

			//get_interface_changeable_data(i->name);
			/* ************************************************* */

			//make_ncurses_interface_info_window();
		    }

		/* Refresh info about devices/interfaces in system */
		if (active_window_devices_flag == TRUE)
		    make_ncurses_devices_window();

		wattron (speed_window, COLOR_PAIR(9));

		/* Maximal speed on an interface */
		/* Packets */
		if (i->speed.pss.speed_packets_top > 1)
		    {
    			mvwaddstr (speed_window, 3, 24, "             ");
			mvwprintw (speed_window, 3, 24, "%llu P/s", 
				    i->speed.pss.speed_packets_top);
		    }
		else
		    {
	    		mvwaddstr (speed_window, 3, 24, "             ");
	    		mvwaddstr (speed_window, 3, 24, "0 P/s");
		    }

		if (i->speed.pss.speed_current_packets > 1)
		    {
	    		mvwaddstr (speed_window, 3, 6, "             ");
			mvwprintw (speed_window, 3, 6, "%llu P/s", 
				    i->speed.pss.speed_current_packets);

		    }
		else
		    {
	    		mvwaddstr (speed_window, 3, 6, "             ");
	    		mvwaddstr (speed_window, 3, 6, "0 P/s");
		    }


		/* KBytes */
		if (i->speed.kbs.speed_kbytes_top > 1)
		    {
    			mvwaddstr (speed_window, 2, 24, "             ");
			mvwprintw (speed_window, 2, 24, "%.2f KB/s", 
				    i->speed.kbs.speed_kbytes_top);
		    }
		else if (i->speed.kbs.speed_kbytes_top < 1 && i->speed.kbs.speed_kbytes_top > 0)
		    {
    			mvwaddstr (speed_window, 2, 24, "             ");
			mvwprintw (speed_window, 2, 24, "%d B/s", 
				    (unsigned int)(i->speed.kbs.speed_kbytes_top * 1024));
		    }


		/* Current speed on an interface */
		if (i->speed.kbs.speed_current_kbytes > 1)
		    {
	    		mvwaddstr (speed_window, 2, 6, "             ");
			mvwprintw (speed_window, 2, 6, "%.2f KB/s", 
				    i->speed.kbs.speed_current_kbytes);

		    }
		else if (i->speed.kbs.speed_current_kbytes < 1 && 
			i->speed.kbs.speed_current_kbytes > 0)
		    {
	    		mvwaddstr (speed_window, 2, 6, "             ");
			mvwprintw (speed_window, 2, 6, "%d B/s", 
				    (unsigned int)(i->speed.kbs.speed_current_kbytes * 1024));
		    }
		else
		    {
	    		mvwaddstr (speed_window, 2, 6, "             ");
	    		mvwaddstr (speed_window, 2, 6, "0.00 KB/s");
		    }

		wattron (speed_window, COLOR_PAIR(1));

		/* counter must = 0 in here */
		counter = 0;
	    }

	wrefresh (now_window);
	wrefresh (speed_window);

	pthread_mutex_unlock(&mutex_id);

	if ((nanosleep(&tvn, &tvn)) == EINTR)
	    continue;
    }
	
	/* Do this if shutdown_thread_refresh_stats_window flag is TRUE */
	return(EXIT_SUCCESS);
}


/* Read to table date and search log_file */
int searching(void)
{
	FILE *fp;
	double float_rx_mb = 0.0, float_tx_mb = 0.0;
	int k = 0, dates_records = 0;
	char date_from[10];
	char date_to[10];

	pthread_mutex_lock(&mutex_id);

	/* ! This is for bug report ! */
	strcpy (running_function, "searching()");

	make_ncurses_search_window();

	searching_flag = TRUE;
	active_window_other_data_flag = FALSE;

	wattron (search_window, COLOR_PAIR(1));
	wattroff (search_window, A_UNDERLINE);
	mvwaddstr (search_window, 6, 25, "             ");
	mvwaddstr (search_window, 7, 25, "             ");
	wattron (search_window, COLOR_PAIR(2));
	mvwaddstr (search_window, 2, 20, "    -  -  ");
	mvwaddstr (search_window, 4, 20, "    -  -  ");
	wmove (search_window, 2, 20);
	waddch (search_window, '_');
	wmove (search_window, 2, 20);
	move (4, 58);
	wrefresh (search_window);
	noecho();
	keypad(stdscr, KEYPAD_OFF);
	/*curs_set(VISIBLE);*/

	pthread_mutex_unlock(&mutex_id);

	top:
	for (k = 0; k < 10; k++)
	    {
		head:
		if (k == 4)
		    {
			date_from[k] = '-';
			move (4, 63);
			wmove (search_window, 2, 25);

			pthread_mutex_lock(&mutex_id);
			wrefresh (search_window);
			pthread_mutex_unlock(&mutex_id);

			k++;
		    }		    
		if (k == 7)
		    {
			date_from[k] = '-';
			move (4, 66);
			wmove (search_window, 2, 28);

			pthread_mutex_lock(&mutex_id);
			wrefresh (search_window);
			pthread_mutex_unlock(&mutex_id);

			k++;
		    }		    
		if ((date_from[k] = getch()) == 9)
		    break;
		if (date_from[k] == 'q' || date_from[k] == 'Q')
		    quit_confirmation();
		if ((isalpha(date_from[k])) != 0 || (ispunct(date_from[k])) != 0 
		    || (isspace(date_from[k])) != 0)
		    goto head;
		if (date_from[k] == KEY_BKSP)
		    {
			if (k == 1) 
			    {
				wmove (search_window, 2, 20);
				waddch (search_window, '_');
				wmove (search_window, 2, 20);
			    }
			if (k == 0)
			    goto top;
			if (k == 5)
			    {
				k = k - 2;
				mvwaddstr (search_window, 2, 20 + k, " ");
				wmove (search_window, 2, 20 + k);

				pthread_mutex_lock(&mutex_id);
				wrefresh (search_window);
				pthread_mutex_unlock(&mutex_id);

				goto head;
			    }
			if (k == 8)
			    {
				k = k - 2;
				mvwaddstr (search_window, 2, 20 + k, " ");
				wmove (search_window, 2, 20 + k);

				pthread_mutex_lock(&mutex_id);
				wrefresh (search_window);
				pthread_mutex_unlock(&mutex_id);

				goto head;
			    }
			if (k != 1)
			    {
				mvwaddstr (search_window, 2, 20 + (k - 1), " ");
				wmove (search_window, 2, 20 + (k - 1));
			    }

			pthread_mutex_lock(&mutex_id);
			wrefresh (search_window);
			pthread_mutex_unlock(&mutex_id);

			k--;
			goto head;
		    }
		if (date_from[k] == KEY_ESC)
		    {
			mvwaddstr (search_window, 2, 20, "    -  -  ");
			wmove (search_window, 2, 20);

			pthread_mutex_lock(&mutex_id);
			wrefresh (search_window);
			pthread_mutex_unlock(&mutex_id);

			goto end;
		    }
		if ((iscntrl(date_from[k])) != 0) 
		    goto head;

		if (date_from[k] == '\n')
		    k--;
		else
		    waddch (search_window, date_from[k]);

		pthread_mutex_lock(&mutex_id);
		wrefresh (search_window);
		pthread_mutex_unlock(&mutex_id);
	    }

	    date_from[k] = '\0';

	    wmove (search_window, 4, 20);
	    waddch (search_window, '_');
	    wmove (search_window, 4, 20);

	    pthread_mutex_lock(&mutex_id);
	    wrefresh (search_window);
	    pthread_mutex_unlock(&mutex_id);

	    for (k = 0; k < 10; k++)
		{
		    head2:
		    if (k == 4)
			{
			    date_to[k] = '-';
			    move (6, 63);
			    wmove (search_window, 4, 25);
			    
			    pthread_mutex_lock(&mutex_id);
			    wrefresh (search_window);
			    pthread_mutex_unlock(&mutex_id);

			    k++;
			}		    
		    if (k == 7)
			{
			    date_to[k] = '-';
			    move (6, 66);
			    wmove (search_window, 4, 28);

			    pthread_mutex_lock(&mutex_id);
			    wrefresh (search_window);
			    pthread_mutex_unlock(&mutex_id);

			    k++;
			}		    
		    if ((date_to[k] = getch()) == 9)
			break;
		    if (date_to[k] == 'q' || date_to[k] == 'Q')
			quit_confirmation();
		    if ((isalpha(date_from[k])) != 0 || 
			(ispunct(date_from[k])) != 0 || 
			(isspace(date_from[k])) != 0)
			    goto head;
		    if (date_to[k] == KEY_BKSP)
			{
			    if (k == 1) 
				{
				    wmove (search_window, 4, 20);
				    waddch (search_window, '_');
				    wmove (search_window, 4, 20);
				}
			    if (k == 0)
				{
				    k = 9;
				    mvwaddstr (search_window, 2, 20 + k, " ");
				    mvwaddstr (search_window, 4, 20, " ");
				    wmove (search_window, 2, 20 + k);

				    pthread_mutex_lock(&mutex_id);
				    wrefresh (search_window);
				    pthread_mutex_unlock(&mutex_id);

				    goto head;
				}
			    if (k == 5)
				{
				    k = k - 2;
				    mvwaddstr (search_window, 4, 20 + k, " ");
				    wmove (search_window, 4, 20 + k);

				    pthread_mutex_lock(&mutex_id);
				    wrefresh (search_window);
				    pthread_mutex_unlock(&mutex_id);

				    goto head2;
				}
			    if (k == 8)
				{
				    k = k - 2;
				    mvwaddstr (search_window, 4, 20 + k, " ");
				    wmove (search_window, 4, 20 + k);

				    pthread_mutex_lock(&mutex_id);
				    wrefresh (search_window);
				    pthread_mutex_unlock(&mutex_id);

				    goto head2;
				}
			    if (k != 1)
				{
				    mvwaddstr (search_window, 4, 20 + (k - 1), " ");
				    wmove (search_window, 4, 20 + (k - 1));
				}

			    pthread_mutex_lock(&mutex_id);
			    wrefresh (search_window);
			    pthread_mutex_unlock(&mutex_id);

			    k--;
			    goto head2;
			}
		    if (date_to[k] == KEY_ESC)
			{
			    mvwaddstr (search_window, 4, 20, "    -  -  ");
		    	    wmove (search_window, 4, 20);

			    pthread_mutex_lock(&mutex_id);
			    wrefresh (search_window);
			    pthread_mutex_unlock(&mutex_id);

			    goto end;
			}
		    if (date_to[k] == '\n')
			k--;
		    else
			waddch (search_window, date_to[k]);
		    
		    pthread_mutex_lock(&mutex_id);
		    wrefresh (search_window);
		    pthread_mutex_unlock(&mutex_id);
		}
	date_to[k] = '\0';

	check_log_read(log_file);
	fp = fopen(log_file, "r");

	while (fread (&log, sizeof(log), 1, fp) == 1)
	    {
		while (strcmp (log.data, date_from) == 0 && 
			strcmp (log.interface, i->name) == 0)
		    {
			dates_records = log.record;
			goto found;
		    }
	    }

	found:
	
	/* 52 bytes have struct log (one record) */
	fseek (fp, -52L, SEEK_CUR);
	
	while (fread (&log, sizeof(log), 1, fp) == 1)
	    {
		if (strcmp (log.data, date_to) != 0 && 
		    strcmp (log.interface, i->name) == 0 && 
		    (dates_records == log.record))
		    {
			float_rx_mb += ((double) log.rx) / 2048;
			float_tx_mb += ((double) log.tx) / 2048;
			dates_records++;
		    }
		if (strcmp (log.data, date_to) == 0 && 
		    strcmp (log.interface, i->name) == 0)
		    {
			float_rx_mb += ((double) log.rx) / 2048;
			float_tx_mb += ((double) log.tx) / 2048;
		    }

	    }

	fclose(fp);

	wattron (search_window, A_UNDERLINE);
	wattron (search_window, COLOR_PAIR(1));

	mvwaddstr (search_window, 6, 25, "              ");
	mvwprintw (search_window, 6, 25, "%.2f MB", (float_tx_mb * 2));

	mvwaddstr (search_window, 7, 25, "              ");
	mvwprintw (search_window, 7, 25, "%.2f MB", (float_rx_mb * 2));
	
	mvwaddstr (search_window, 3, 14, "");
	wattron (search_window, A_NORMAL);

	pthread_mutex_lock(&mutex_id);
	wrefresh (search_window);
	pthread_mutex_unlock(&mutex_id);
	    
	end:

	searching_flag = FALSE;

	keypad(stdscr, KEYPAD_ON);

	/*curs_set(INVISIBLE);*/

	return(EXIT_SUCCESS);
}


/* Sum of date from log file */
int total_bytes(void)
{
	FILE *fp;
	unsigned long long int_rx_suma_b = 0, int_tx_suma_b = 0;
	int rec = 0, device_in_log = 0;

	/* ! This is for bug report ! */
	strcpy (running_function, "total_bytes()");

	check_log_read(log_file);
	fp = fopen(log_file, "r");

	while (fread (&log, sizeof(log), 1, fp) == 1)
	if (strcmp (log.interface, i->name) == 0)
	    {
		int_tx_suma_b += log.tx;
		int_rx_suma_b += log.rx;
		rec++;
		device_in_log = 1;
	    }


	/* If not find entered interface in log file then clean date's infos */
	if (device_in_log != 1)
	    {
		wattroff (total_window, A_UNDERLINE);
		wattron (total_window, COLOR_PAIR(1));
		mvwaddstr (total_window, 1, 5, "          ");
		mvwaddstr (total_window, 1, 20, "          ");
	    }
	else
	    {
		wattroff (total_window, A_UNDERLINE);
		wattron (total_window, COLOR_PAIR(9));
		
		/* Rewind reading at last record in log file */
		fseek (fp, -52L, SEEK_END);
		while (fread (&log, sizeof(log), 1, fp) == 1)
		    {
			/* Print data member end break */
			mvwprintw (total_window, 1, 20, "%s", log.data);
			wrefresh (total_window);
			break;
		    }

		/* Rewind reading at first record in log file */
		fseek (fp, 0L, SEEK_SET);
		while (fread (&log, sizeof(log), 1, fp) == 1)
		    {
			/* Print data member end break */
			mvwprintw (total_window, 1, 5, "%s", log.data);
			wrefresh (total_window);
			break;
		    }
	    }

	fclose(fp);

	if (device_in_log != 1)
	    {
	    	int_tx_suma_b = 0;
		int_rx_suma_b = 0;
	    }

	wattroff (total_window, A_UNDERLINE);
	wattron (total_window, COLOR_PAIR(1));
	mvwaddstr (total_window, 2, 20, "              ");
	mvwaddstr (total_window, 3, 20, "              ");
	mvwaddstr (total_window, 4, 20, "              ");
	wattron (total_window, A_UNDERLINE);


	if (int_tx_suma_b > 1024)
	    mvwprintw (total_window, 2, 20, "%.2f KB", 
			((double) int_tx_suma_b) / 1024);
	else if ((int_tx_suma_b / 1024) > 1024)
	    mvwprintw (total_window, 2, 20, "%.2f MB", 
			((double) int_tx_suma_b) / 2048);
	else
	    mvwprintw (total_window, 2, 20, "%llu B", int_tx_suma_b);


	if (int_rx_suma_b > 1024)
	    mvwprintw (total_window, 3, 20, "%.2f KB", 
			((double) int_rx_suma_b) / 1024);
	else if ((int_rx_suma_b / 1024) > 1024)
	    mvwprintw (total_window, 3, 20, "%.2f MB", 
			((double) int_rx_suma_b) / 2048);
	else
	    mvwprintw (total_window, 3, 20, "%llu B", int_rx_suma_b);


	mvwprintw (total_window, 4, 20, "%d", rec);

	wattron (total_window, A_NORMAL);
	wrefresh (total_window);

	return(EXIT_SUCCESS);
}


void no_data_in_log_for_interface_msg(void)
{
	printf ("       Sorry but no data available in your "_STRING_WHITE_
		"\n       log file!!! for "_STRING_WHITE_" interface :-(\n\n", 
		log_file, i->name);
}


/* Summary bytes for interface from log_file */
int summary_bytes(void)
{
	FILE *fp;
	unsigned long long int_rx_suma_b = 0, int_tx_suma_b = 0;

	/* ! This is for bug report ! */
	strcpy (running_function, "summary_bytes()");

	check_log_read(log_file);
	check_if_old_log();
	
	if (log_file_size == 0)
	    return(EXIT_FALSE);

	fp = fopen(log_file, "r");
    
	data_is_available = FALSE;
	
	while (fread (&log, sizeof(log), 1, fp) == 1)
	if (strcmp (log.interface, i->name) == 0)
	    {
		int_rx_suma_b += log.rx;
		int_tx_suma_b += log.tx;
		
		data_is_available = TRUE;
	    }
        
	fclose(fp);

	printf ("Summary Bytes for: "_STRING_WHITE_"\n\n", i->name);

	if (data_is_available == FALSE)
	    no_data_in_log_for_interface_msg();
	else
	    {
		printf ("TX:   %llu B\n", int_tx_suma_b);
		printf ("RX:   %llu B\n\n", int_rx_suma_b);
	    }

	exit_prog(EXIT_SUCCESS);

	return(EXIT_SUCCESS);
}


/* Read data from log_file */
int read_log(void)
{
	FILE *fp;
	
	/* ! This is for bug report ! */
	strcpy (running_function, "read_log()");

	check_log_read(log_file);
	check_if_old_log();

	if (log_file_size == 0)
	    return(EXIT_FALSE);

	fp = fopen(log_file, "r");

	printf ("Reading Records for: "_STRING_WHITE_"\n\n", i->name);

	data_is_available = FALSE;

	while (fread (&log, sizeof(log), 1, fp) == 1)
	if (strcmp (log.interface, i->name) == 0)
	    {
		printf ("Date: %s\n", log.data);

		printf ("TX:   %llu B\n", log.tx);
		printf ("RX:   %llu B\n", log.rx);

		printf ("Rec:  %lu\n\n", log.record);
		
		data_is_available = TRUE;
	    }

	fclose(fp);
	
	if (data_is_available == FALSE)
	    no_data_in_log_for_interface_msg();


	exit_prog(EXIT_SUCCESS);

	return(EXIT_SUCCESS);
}


/* Read data from log file and write output to normal (text) file */
int write_log_as_text(void)
{
	FILE *fp;
	
	/* ! This is for bug report ! */
	strcpy (running_function, "write_log_as_text()");

	check_log_read(log_file);
	check_if_old_log();

	if (log_file_size == 0)
	    return(EXIT_FALSE);

	fp = fopen(log_file, "r");

	printf ("Reading Records for: "_STRING_WHITE_"\n\n", i->name);

	data_is_available = FALSE;

	while (fread (&log, sizeof(log), 1, fp) == 1)
	if (strcmp (log.interface, i->name) == 0)
	    {
		printf ("Date: %s\n", log.data);

		printf ("TX:   %llu B\n", log.tx);
		printf ("RX:   %llu B\n", log.rx);

		printf ("Rec:  %lu\n\n", log.record);
		
		data_is_available = TRUE;
	    }

	fclose(fp);
	
	if (data_is_available == FALSE)
	    no_data_in_log_for_interface_msg();


	exit_prog(EXIT_SUCCESS);

	return(EXIT_SUCCESS);
}


void log_file_older_than_ver4_msg(void)
{
	printf(STRING_FAIL" "_COLOR_RED_"< ver 0.4"_COLOR_END_"\n\n");
	printf ("    	Log file older than 0.4 can't convert to 0.9 :-(\n"
		"    	Sorry but your "_STRING_WHITE_" log file is too old!!!\n"
		"    	You must erase it. Please be sure that you don't write\n" 
		"    	in log file with bcount older than %s. This programm\n"
		"    	convert log file written by bcount >= 0.4 <= 0.8 to new\n"
		"    	log file supported by bcount >= 0.9\n\n", 
		log_file, VERSION);
}


/* Check if we must delete old log file :-( */
int check_if_old_log(void)
{
	FILE *current_log, *temp_log;
	struct stat *st = (struct stat *) xmalloc (sizeof(struct stat));
	struct passwd *pwd = (struct passwd *) xmalloc (sizeof(struct passwd));
	char alpha[2], *temp_log_file;
	char vtfile[strlen(log_file) + 5];
	char *vfile = vtfile;
	int rec = 0, check = 0, ch = 0, error = 0;
 
	/* ! This is for bug report ! */
	strcpy (running_function, "check_if_old_log()");

	temp_log_file = generate_temp_file(default_prefix);

	strcpy(vfile, log_file);
	strcat(vfile, ".old");

	if ((current_log = fopen(log_file, "r")) == NULL)
	    {
		check_log_write(log_file);
		return(EXIT_SUCCESS);
	    }

	get_file_info(log_file);

	printf ("+ Checking log file: ");

	/* First we check if log file != 0 */
	if (log_file_size == 0)
	    {
		printf(_COLOR_RED_"Null size!"_COLOR_END_"\n");
		printf("    File "_STRING_WHITE_" have 0 bytes.\n\n", log_file);
		fclose(current_log);

		return(EXIT_SUCCESS);
	    }

	/* Record counter reset */
	rec = 0;

	/* Jump to beginning record */
	fseek (current_log, 0L, SEEK_SET);
	
	while (fread (&log, sizeof(log), 1, current_log) == 1)
	    rec++;

	/* Check if log file is = 0.9 */
	if (log_file_size >= 52)
	    {
		if (log_file_size == 52 || (check = (log_file_size / rec)) == 52)
		    {
			printf(STRING_OK" "_COLOR_GREEN_"= ver 0.9"_COLOR_END_"\n");
			printf ("    Struct of log file is Ok :-)\n\n");

			fclose(current_log);

			return(EXIT_SUCCESS);
		    }
	    }
	    
	/* Record counter reset */
	rec = 0;

	/* Jump to beginning record */
	fseek (current_log, 0L, SEEK_SET);
	
	while (fread (&log_old, sizeof(log_old), 1, current_log) == 1)
	    rec++;

	/* Check older log version < 0.4 */
	if (log_file_size == 40)
	    {
		log_file_older_than_ver4_msg();
		fclose(current_log);

		exit_prog(EXIT_FALSE);
	    }
	else if (log_file_size != 40)
	    {
		rec++;
	
		if ((check = (log_file_size / rec)) == 40)
		    {
			log_file_older_than_ver4_msg();
			fclose(current_log);

			exit_prog(EXIT_FALSE);
		    }

		rec--;
	    }


	/* Check older log version >= 0.4 */
	if (log_file_size == 44 || (check = (log_file_size / rec)) == 44)
	    {
		printf(STRING_OK" "_COLOR_GREEN_">= ver 0.4"_COLOR_END_"\n");

		fclose(current_log);

		current_log = fopen(log_file, "r");
		printf ("    Log file newer than 0.3 but older than 0.9.\n\n");

		printf ("+ Converting...\n");
		printf ("  -> Touching temporary file...\n");
		while ((access(temp_log_file, R_OK)) == 0)
		    {
			if (unlink(temp_log_file) == -1)
			    {
				if (errno == EPERM || errno == EACCES)
				    printf ("       "_STRING_RED_": Can't "
					    "remove "_STRING_WHITE_"\n"
					    , strerror(errno), temp_log_file);
				else
				    printf ("       "_STRING_RED_"\n"
					    , strerror(errno));

				temp_log_file[strlen(temp_log_file) - 1] = 
				(alpha[0] = 48 + ++ch); /* char (48) = 0 */
			    }
		    }
				    

		printf("       Touching "_STRING_WHITE_" new temporary "
			"file... ", temp_log_file);

		if ((temp_log = fopen(temp_log_file, "a")) == NULL)
		    {
			printf(_STRING_RED_"\n", STRING_FAIL);
			goto hell;
		    }
		else
		    printf(_STRING_WHITE_"\n", STRING_OK);
		    
		while (fread (&log_old, sizeof(log_old), 1, current_log) == 1)
		    {
			strcpy(log.data, log_old.data);
			strcpy(log.interface, log_old.interface);			
			log.rx = log_old.rx;			
			log.tx = log_old.tx;			
			log.record = log_old.record;			
			fwrite (&log, sizeof(log), 1, temp_log);    
		    }

		fclose(current_log);
		fclose(temp_log);

		while (access(vfile, R_OK) == 0)
		    {
			printf("       Log backup file exist... "
				"try to erase it... ");
			if (unlink(vfile) == -1)
			    {
				printf(_STRING_RED_"\n", STRING_FAIL);
			    	if (errno == EPERM || errno == EACCES)
				    printf ("       "_STRING_RED_": Can't "
					    "remove "_STRING_WHITE_"\n"
					    , strerror(errno), vfile);
				else
				    printf ("       "_STRING_RED_"\n"
					    , strerror(errno));

				lstat(vfile, st);
				pwd = getpwuid(st->st_uid);
				if (st->st_uid == 0) {
				    printf ("       This file touching by "
					    _STRING_WHITE_" your uid "
					    "don't have access to change it.\n"
					    , pwd->pw_name);
				    }
				else if (st->st_uid != 0) {
				    printf ("       This file touching by "
					    _STRING_WHITE_" your uid "
					    "don't have access\n"
					    "       to change it. Only "
					    _STRING_WHITE_
					    " or "_COLOR_WHITE_"root"_COLOR_END_
					    " can do this\n", pwd->pw_name
					    , pwd->pw_name);
				    }

				free(st);
				free(pwd);
				error = 1;
			    }
			else
			    printf(_STRING_WHITE_"\n", STRING_OK);

		    }
		
		printf("  -> Moving "_STRING_WHITE_" to "
			_STRING_WHITE_"... ", log_file, vfile);

		if (rename(log_file, vfile) == -1)
		    {
			error = 1;
		    }
		else		    
		    printf(_STRING_WHITE_"\n", STRING_OK);


		printf("  -> Moving "_STRING_WHITE_" to "
			_STRING_WHITE_"... "
			, temp_log_file, log_file);

		if ((temp_log = fopen(temp_log_file, "r")) == NULL)
		    {
			printf(_STRING_RED_"\n", STRING_FAIL);
			if (errno == EPERM || errno == EACCES)
			    printf ("         "_STRING_RED_": Can't read "
				    _STRING_WHITE_"\n"
				    , strerror(errno), temp_log_file);
			else if (errno == ENOENT)
			    printf ("         "_STRING_RED_": file "
				    _STRING_WHITE_" don't exist\n"
				    , strerror(errno), temp_log_file);
			else
			    printf ("         "_STRING_RED_"\n"
				    , strerror(errno));

			error = 1;
		    }

		if ((current_log = fopen(log_file, "a")) == NULL)
		    {
			printf(_STRING_RED_"\n", STRING_FAIL);
			if (errno == EPERM || errno == EACCES)
			    printf ("         "_STRING_RED_": Can't write "
				    _STRING_WHITE_"\n", strerror(errno), log_file);
			else
			    printf ("         "_STRING_RED_"\n"
				    , strerror(errno));

			error = 1;
		    }

		if (error == 1)
		    goto hell;
				    
		while ((ch = fgetc(temp_log)) != EOF)
		    fputc(ch, current_log);

		printf(_STRING_WHITE_"\n", STRING_OK);
		printf ("  -> Done.\n\n");

		fclose(current_log);
		fclose(temp_log);

		/* Remove temp file */
		printf ("  -> Removing temporary file...\n");

		if (unlink(temp_log_file) == -1)
		    {
			if (errno == EPERM || errno == EACCES)
			    printf ("       "_STRING_RED_": Can't "
				    "remove "_STRING_WHITE_"\n"
				    , strerror(errno), temp_log_file);
			else if (errno == ENOENT)
			    printf ("       "_STRING_RED_": File\n"
				    _STRING_WHITE_" don't exist!\n"
				    , strerror(errno), temp_log_file);
			else
			    printf ("       "_STRING_RED_"\n"
				    , strerror(errno));
		    }

		printf ("       Removing temporary file "_STRING_WHITE_"...",
			temp_log_file);

		if ((access(temp_log_file, R_OK)) == 0)
		    printf(_STRING_RED_"\n\n", STRING_FAIL);
		else
		    printf(_STRING_WHITE_"\n\n", STRING_OK);

		/* Get new log file info */
		if (get_file_info(log_file) == -1)
		    {
			printf("       "_STRING_RED_": Error when get info "
				"about "_STRING_WHITE_"\n"
				, strerror(errno), log_file);
			goto hell;
		    }

		printf ("+ New log file info:\n");
		printf ("    log name    = %s\n", log_file);
		printf ("    log size    = %d Bytes\n", log_file_size);
		printf ("    record size = %d Bytes\n", log_file_size / rec );
		printf ("    records     = %d\n\n", rec);
		
		hell:

		press_any_key();

		return(EXIT_SUCCESS);
	    }
	/* else log file is mismashed */
	else
	    {
		printf(_COLOR_RED_"Unknown!"_COLOR_END_"\n\n");
		printf ("       Oops... something wrong with your "_STRING_WHITE_".\n"
			"       Please be sure that you don't write in log\n"
			"       file with bcount older than %s.\n"
			"       Sorry but you must erase this file! :-(\n\n", 
			log_file, VERSION);

		fclose(current_log);

		exit_prog(EXIT_FALSE);
	    }

	return(EXIT_FALSE);
}


/* Gets number of records from log_file */
int get_nr_record(void)
{
	FILE *fp;

	/* ! This is for bug report ! */
	strcpy (running_function, "get_nr_record()");

	check_log_read(log_file);
	record_number = 0;
	fp = fopen(log_file, "r");

	while (fread (&log, sizeof(log), 1, fp) == 1)
	    record_number++;

	fclose(fp);
	return(EXIT_SUCCESS);
}


/* Write data to log_file */
int write_log(void)
{
	FILE *fp;

	/* ! This is for bug report ! */
	strcpy (running_function, "write_log()");

	/* Check if we can write into log file */
	check_log_write(log_file);

	/* Check if we don't have old kernel */
	kernel_release();

	/* We don't need old log file */
	check_if_old_log();

	get_nr_record();

	fp = fopen(log_file, "a");

	/* ********************************************* */
	free_interface_info_struct();

	i = (struct interface_info *) 
	    xmalloc (sizeof (struct interface_info));

	get_interface_data(i->name);
	/* ********************************************* */

	strcpy(log.data, get_localdate());
	strcpy(log.interface, i->name);
	log.rx = i->stats.rx_bytes;
	log.tx = i->stats.tx_bytes;
	log.record = (long) (record_number + 1);

	fwrite (&log, sizeof(log), 1, fp);

	printf ("%s: Saved ", i->name);

	if (!strcmp (data_type, MBYTES))
	    {
		printf ("RX:%.2f MB and ", i->stats.rx_mbytes);
		printf ("TX:%.2f MB in log file.\n", i->stats.tx_mbytes);
	    }
	else if (!strcmp (data_type, KBYTES))
	    {
		printf ("RX:%.2f KB and ", i->stats.rx_kbytes);
		printf ("TX:%.2f KB in log file.\n", i->stats.tx_kbytes);
	    }
	else if (!strcmp (data_type, BYTES))
	    {
		printf ("RX:%llu B and ", i->stats.rx_bytes);
		printf ("TX:%llu B in log file.\n", i->stats.tx_bytes);
	    }
	else
	    {
		printf ("RX:%llu B and ", i->stats.rx_bytes);
		printf ("TX:%llu B in log file.\n", i->stats.tx_bytes);
	    }

	printf ("Syncing %s ", log_file);
	if (fflush (fp) == 0)
	    printf ("...Done\n");
	fclose (fp);
	
	exit_prog(EXIT_SUCCESS);
}


/* Checking if log file exist */
void check_log_read(char *file_for_read)
{
	/* ! This is for bug report ! */
	strcpy (running_function, "check_log_read()");

	if ((access (file_for_read, R_OK)) != 0)
	    read_log_error();
}


/* Checking if we can open log file in append mode
    If file don't exist then will be created */
void check_log_write(char *file_for_write)
{
	FILE *fp2;
 
	/* ! This is for bug report ! */
	strcpy (running_function, "check_log_write()");

	if ((fp2 = fopen (file_for_write, "a")) == NULL)
	    write_log_error();
	
	fclose(fp2);
}


/* Interface what we wish */
int choice_interface(void)
{
	int k = 0;
	char temp_interface[11];
	
	pthread_mutex_lock(&mutex_id);

	/* ! This is for bug report ! */
	strcpy (running_function, "choice_interface()");

	choice_if_flag = TRUE;
	
	wattron (choice_if_window, A_NORMAL | COLOR_PAIR(2));
	mvwaddstr (choice_if_window, 1, 13, "          ");
	move (17, 51);
	wmove (choice_if_window, 1, 13);
	waddch (choice_if_window, '_');
	wmove (choice_if_window, 1, 13);
	wrefresh(choice_if_window);
	noecho();
	keypad(stdscr, KEYPAD_OFF);
	/*curs_set(VISIBLE);*/

	pthread_mutex_unlock(&mutex_id);

	interface_top:

	for (k = 0; k < 10; k++)
	    {
		interface_head:
		if ((temp_interface[k] = getch()) == 9)
		    break;
		if (temp_interface[k] == KEY_BKSP)
		    {
			if (k == 1) 
			    {
				wmove (choice_if_window, 1, 13);
				waddch (choice_if_window, '_');
				wmove (choice_if_window, 1, 13);
			    }

			if (k == 0)
			    goto interface_top;
			    
			if (k != 1) 
			    {
				mvwaddstr (choice_if_window, 1, 13 + (k - 1), " ");
				wmove (choice_if_window, 1, 13 + (k - 1));
			    }

			pthread_mutex_lock(&mutex_id);
			wrefresh (choice_if_window);
			pthread_mutex_unlock(&mutex_id);

			k--;
			goto interface_head;
		    }
		if (temp_interface[k] == KEY_ESC)
		    {
			mvwaddstr (choice_if_window, 1, 13, "          ");

			pthread_mutex_lock(&mutex_id);
			wrefresh (choice_if_window);
			pthread_mutex_unlock(&mutex_id);

			goto interface_end;
		    }
		if (temp_interface[k] == '\n')
		    break;
            
		waddch (choice_if_window, temp_interface[k]);

		pthread_mutex_lock(&mutex_id);
		wrefresh (choice_if_window);
		pthread_mutex_unlock(&mutex_id);
	    }

	temp_interface[k] = '\0';
	
	/*curs_set(INVISIBLE);*/

	pthread_mutex_lock(&mutex_id);

	if (search_for_interface(temp_interface) != INTERFACE_NOT_FOUND)
	    {
		/* ******************************************************** */
		/* Try to free allocated memory for `struct interface_info' */
		free_interface_info_struct();	

		i = (struct interface_info *) 
		    xmalloc (sizeof (struct interface_info));

		/* Get interface info ... like ip, netmask ...etc. */
		get_interface_data(temp_interface);
		/* ******************************************************** */

		total_bytes();

		make_ncurses_interface_window();

		if (active_window_interface_info_flag == TRUE)
		    make_ncurses_interface_info_window();
		
		/* Reset last remember counter */
		clear_interface_speed_flag = TRUE; /* This will be reset in
						    pthreads_interface_speed() */

		/* This is required if in Now Window remain shit */
		mvwaddstr (now_window, 2, 20, "              ");
		mvwaddstr (now_window, 3, 20, "              ");
		mvwaddstr (now_window, 4, 20, "              ");
		mvwaddstr (now_window, 5, 20, "              ");
		mvwaddstr (now_window, 6, 20, "              ");
		mvwaddstr (now_window, 7, 20, "              ");
		wrefresh (now_window);

		mvwaddstr (choice_if_window, 1, 13, "          ");
		wrefresh (choice_if_window);
	    }
	else
	    {
		wattron (status_window, COLOR_PAIR(2));
		fill_panel (status_window);
		mvwprintw (status_window, 0, 0, " - Interface <%s> was not "
			    "found on this system or is down! :-(", 
			    temp_interface);

		wrefresh (status_window);

		sleep(4);

		mvwaddstr (choice_if_window, 1, 13, "          ");
		wrefresh (choice_if_window);

		wattron (status_window, COLOR_PAIR(2));
		fill_panel (status_window);
		mvwprintw (status_window, 0, 0, " - Log file: %s "
			    "(%d Bytes | UID: %d GID: %d) ", 
			    log_file, log_file_size, file_uid, file_gid);

		wrefresh (status_window);
	    }	    

	pthread_mutex_unlock(&mutex_id);
	
	interface_end:

	/*curs_set(INVISIBLE);*/
	choice_if_flag = FALSE;

	keypad(stdscr, KEYPAD_ON);

	return(EXIT_SUCCESS);
}


/* If we received couple of bytes then we got mail */
int make_mail(void)
{
	int k = 0;

	pthread_mutex_lock(&mutex_id);

	/* ! This is for bug report ! */
	strcpy (running_function, "make_mail()");

	active_window_other_data_flag = FALSE;
	email_flag = TRUE;

	wattron (mail_status_window, A_NORMAL | COLOR_PAIR(2));
	mvwaddstr (mail_status_window, 0, 1, "       ");
	make_ncurses_mail_window();
	wrefresh(mail_status_window);
	noecho();
	keypad(stdscr, KEYPAD_OFF);
	/*curs_set(VISIBLE);*/
	pthread_mutex_unlock(&mutex_id);

	mail_check_head_1:
	pthread_mutex_lock(&mutex_id);
	move (4, 56);
	wmove (mail_window, 2, 18);
	waddch (mail_window, '_');
	wmove (mail_window, 2, 18);

	wrefresh (mail_window);
	pthread_mutex_unlock(&mutex_id);

	transmitted_char_check = getch();

	if (transmitted_char_check == 'q' || transmitted_char_check == 'Q')
	    quit_confirmation();

	if (transmitted_char_check == KEY_ESC)
	    {
		pthread_mutex_lock(&mutex_id);
		no_more_mail_flag = TRUE;
		transmitted_flag = FALSE;
		received_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto mail_end;
	    }

	if (transmitted_char_check == '\n')
	    {
		pthread_mutex_lock(&mutex_id);
		wmove (mail_window, 2, 18);
		waddch (mail_window, ' ');
		wmove (mail_window, 2, 18);

		wrefresh (mail_window);

		transmitted_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto mail_check_head_2;
	    }

	if (transmitted_char_check == ' ' || transmitted_char_check == 't' 
	    || transmitted_char_check == 'T')
	    {
		pthread_mutex_lock(&mutex_id);
		waddch (mail_window, 'X');

		wrefresh (mail_window);

		transmitted_flag = TRUE;
		received_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto mail_top;
	    }
	else
	    goto mail_check_head_1;


	mail_check_head_2:

	pthread_mutex_lock(&mutex_id);
	move (4, 73);
	wmove (mail_window, 2, 35);
	waddch (mail_window, '_');
	wmove (mail_window, 2, 35);

	wrefresh (mail_window);
	pthread_mutex_unlock(&mutex_id);

	received_char_check = getch();

	if (received_char_check == 'q' || received_char_check == 'Q')
	    quit_confirmation();

	if (received_char_check == KEY_ESC)
	    {
		pthread_mutex_lock(&mutex_id);
		no_more_mail_flag = TRUE;
		transmitted_flag = FALSE;
		received_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto mail_end;
	    }

	if (received_char_check == '\n')
	    {
		pthread_mutex_lock(&mutex_id);
		wmove (mail_window, 2, 35);
		waddch (mail_window, ' ');
		wmove (mail_window, 2, 35);

		wrefresh (mail_window);

		received_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto mail_check_head_1;
	    }

	if (received_char_check == ' ' || received_char_check == 'r' 
	    || received_char_check == 'R')
	    {
		pthread_mutex_lock(&mutex_id);
		waddch (mail_window, 'X');

		wrefresh (mail_window);

		transmitted_flag = FALSE;
		received_flag = TRUE;
		pthread_mutex_unlock(&mutex_id);
		goto mail_top;
	    }
	else
	    goto mail_check_head_2;


	mail_top:

	pthread_mutex_lock(&mutex_id);
	move (6, 43);
	wmove (mail_window, 4, 5);
	waddch (mail_window, '_');
	wmove (mail_window, 4, 5);

	wrefresh (mail_window);
	pthread_mutex_unlock(&mutex_id);

	mail_top_1:
	for (k = 0; k < 6; k++)
	    {
		mail_head_1:
		if ((get_mail_digit[k] = getch()) == 5)
		    break;
		if (get_mail_digit[k] == 'q' || get_mail_digit[k] == 'Q')
		    quit_confirmation();
		if (get_mail_digit[k] == KEY_BKSP)
		    {
			if (k == 1)
			    {
				wmove (mail_window, 4, 5);
				waddch (mail_window, '_');
				wmove (mail_window, 4, 5);
			    }

			if (k == 0)
			    goto mail_top_1;

			if (k != 1)
			    {
				mvwaddstr (mail_window, 4, 5 + (k - 1), " ");
				wmove (mail_window, 4, 5 + (k - 1));
			    }

			pthread_mutex_lock(&mutex_id);
			wrefresh (mail_window);
			pthread_mutex_unlock(&mutex_id);

			k--;
			goto mail_head_1;
		    }
		if (get_mail_digit[k] == KEY_ESC)
		    {
			mvwaddstr (mail_window, 4, 5, "      ");

			pthread_mutex_lock(&mutex_id);
			wrefresh (mail_window);
			pthread_mutex_unlock(&mutex_id);

			no_more_mail_flag = TRUE;
			goto mail_end;
		    }
		if (get_mail_digit[k] == '\n')
		    break;
		if (isdigit(get_mail_digit[k]) == 0)
		    goto mail_head_1;
            
		waddch (mail_window, get_mail_digit[k]);

		pthread_mutex_lock(&mutex_id);
		wrefresh (mail_window);
		pthread_mutex_unlock(&mutex_id);

	    }

	get_mail_digit[k] = '\0';

	pthread_mutex_lock(&mutex_id);
	move (8, 40);
	wmove (mail_window, 6, 2);
	waddch (mail_window, '_');
	wmove (mail_window, 6, 2);

	wrefresh (mail_window);
	pthread_mutex_unlock(&mutex_id);

	mail_top_2:
	for (k = 0; k < 36; k++)
	    {
		mail_head_2:
		if ((get_mail_address[k] = getch()) == 35)
		    break;
		if (get_mail_address[k] == KEY_BKSP)
		    {
			if (k == 1)
			    {
				wmove (mail_window, 6, 2);
				waddch (mail_window, '_');
				wmove (mail_window, 6, 2);
			    }
			if (k == 0)
                	    goto mail_top_2;

			if (k != 0)
			    {
				mvwaddstr (mail_window, 6, 2 + (k - 1), " ");
				wmove (mail_window, 6, 2 + (k - 1));
			    }

			pthread_mutex_lock(&mutex_id);
			wrefresh (mail_window);
			pthread_mutex_unlock(&mutex_id);

			k--;
			goto mail_head_2;
		    }
		if (get_mail_address[k] == KEY_ESC)
		    {
			mvwaddstr (mail_window, 6, 2, 
				    "                                 ");

			pthread_mutex_lock(&mutex_id);
			wrefresh (mail_window);
			pthread_mutex_unlock(&mutex_id);

			no_more_mail_flag = TRUE;
			goto mail_end;
		    }

		if (get_mail_address[k] == '\n')
		    break;

		pthread_mutex_lock(&mutex_id);
		waddch (mail_window, get_mail_address[k]);

		wrefresh (mail_window);
		pthread_mutex_unlock(&mutex_id);
	    }

	get_mail_address[k] = '\0';

	no_more_mail_flag = FALSE;

	if (strlen(get_mail_address) > 0 && strlen(get_mail_digit) > 0)
	    {
		wattron (mail_status_window, A_BLINK | COLOR_PAIR(5));
		mvwaddstr (mail_status_window, 0, 1, "Waiting");

		pthread_mutex_lock(&mutex_id);
		wrefresh (mail_status_window);
		pthread_mutex_unlock(&mutex_id);
	    }
      
	mail_end:

	/*curs_set(INVISIBLE);*/
	email_flag = FALSE;

	keypad(stdscr, KEYPAD_ON);

	pthread_mutex_lock(&mutex_id);
	make_ncurses_search_window();
	pthread_mutex_unlock(&mutex_id);

	return(EXIT_SUCCESS);
}


/*  */
int make_shutting_down(void)
{
	int k = 0;

	pthread_mutex_lock(&mutex_id);

	/* ! This is for bug report ! */
	strcpy (running_function, "make_shutting_down()");

	active_window_other_data_flag = FALSE;

	make_ncurses_down_window();
	noecho();
	keypad(stdscr, KEYPAD_OFF);
	/*curs_set(VISIBLE);*/
	pthread_mutex_unlock(&mutex_id);

	down_check_head_1:
	pthread_mutex_lock(&mutex_id);
	move (4, 56);
	wmove (down_window, 2, 18);
	waddch (down_window, '_');
	wmove (down_window, 2, 18);

	wrefresh (down_window);
	pthread_mutex_unlock(&mutex_id);

	reboot_char_check = getch();

	if (reboot_char_check == 'q' || reboot_char_check == 'Q')
	    quit_confirmation();

	if (reboot_char_check == KEY_ESC)
	    {
		pthread_mutex_lock(&mutex_id);
		reboot_system_flag = FALSE;
		shutdown_system_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto down_end;
	    }

	if (reboot_char_check == '\n')
	    {
		pthread_mutex_lock(&mutex_id);
		wmove (down_window, 2, 18);
		waddch (down_window, ' ');
		wmove (down_window, 2, 18);

		wrefresh (down_window);

		reboot_system_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto down_check_head_2;
	    }

	if (reboot_char_check == ' ' || reboot_char_check == 'r' 
	    || reboot_char_check == 'R')
	    {
		pthread_mutex_lock(&mutex_id);
		waddch (down_window, 'X');

		wrefresh (down_window);

		reboot_system_flag = TRUE;
		shutdown_system_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto down_top;
	    }
	else
	    goto down_check_head_1;


	down_check_head_2:

	pthread_mutex_lock(&mutex_id);
	move (4, 73);
	wmove (down_window, 2, 35);
	waddch (down_window, '_');
	wmove (down_window, 2, 35);

	wrefresh (down_window);
	pthread_mutex_unlock(&mutex_id);

	shutdown_char_check = getch();

	if (shutdown_char_check == 'q' || shutdown_char_check == 'Q')
	    quit_confirmation();

	if (shutdown_char_check == KEY_ESC)
	    {
		pthread_mutex_lock(&mutex_id);
		reboot_system_flag = FALSE;
		shutdown_system_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto down_end;
	    }

	if (shutdown_char_check == '\n')
	    {
		pthread_mutex_lock(&mutex_id);
		wmove (down_window, 2, 35);
		waddch (down_window, ' ');
		wmove (down_window, 2, 35);

		wrefresh (down_window);

		shutdown_system_flag = FALSE;
		pthread_mutex_unlock(&mutex_id);
		goto down_check_head_1;
	    }

	if (shutdown_char_check == ' ' || shutdown_char_check == 's' 
	    || shutdown_char_check == 'S')
	    {
		pthread_mutex_lock(&mutex_id);
		waddch (down_window, 'X');

		wrefresh (down_window);

		reboot_system_flag = FALSE;
		shutdown_system_flag = TRUE;
		pthread_mutex_unlock(&mutex_id);
		goto down_top;
	    }
	else
	    goto down_check_head_2;


	down_top:

	pthread_mutex_lock(&mutex_id);
	move (6, 43);
	wmove (down_window, 4, 5);
	waddch (down_window, '_');
	wmove (down_window, 4, 5);

	wrefresh (down_window);
	pthread_mutex_unlock(&mutex_id);

	down_top_1:
	for (k = 0; k < 6; k++)
	    {
		down_head_1:
		if ((get_down_digit[k] = getch()) == 5)
		    break;
		if (get_down_digit[k] == 'q' || get_down_digit[k] == 'Q')
		    quit_confirmation();
		if (get_down_digit[k] == KEY_BKSP)
		    {
			if (k == 1)
			    {
				wmove (down_window, 4, 5);
				waddch (down_window, '_');
				wmove (down_window, 4, 5);
			    }

			if (k == 0)
			    goto down_top_1;

			if (k != 1)
			    {
				mvwaddstr (down_window, 4, 5 + (k - 1), " ");
				wmove (down_window, 4, 5 + (k - 1));
			    }

			pthread_mutex_lock(&mutex_id);
			wrefresh (down_window);
			pthread_mutex_unlock(&mutex_id);

			k--;
			goto down_head_1;
		    }
		if (get_down_digit[k] == KEY_ESC)
		    {
			mvwaddstr (down_window, 4, 5, "      ");

			pthread_mutex_lock(&mutex_id);
			wrefresh (down_window);
			pthread_mutex_unlock(&mutex_id);

			goto down_end;
		    }
		if (get_down_digit[k] == '\n')
		    break;
		if (isdigit(get_down_digit[k]) == 0)
		    goto down_head_1;
            
		waddch (down_window, get_down_digit[k]);

		pthread_mutex_lock(&mutex_id);
		wrefresh (down_window);
		pthread_mutex_unlock(&mutex_id);

	    }

	get_down_digit[k] = '\0';


	if (strlen(get_down_digit) > 0)
	    {
		wattron (mail_status_window, A_BLINK | COLOR_PAIR(5));
		mvwaddstr (mail_status_window, 0, 1, "Waiting");

		pthread_mutex_lock(&mutex_id);
		wrefresh (mail_status_window);
		pthread_mutex_unlock(&mutex_id);
	    }
      
	down_end:

	/*curs_set(INVISIBLE);*/

	keypad(stdscr, KEYPAD_ON);

	pthread_mutex_lock(&mutex_id);
	make_ncurses_search_window();
	pthread_mutex_unlock(&mutex_id);

	return(EXIT_SUCCESS);
}


/* NIN */



